#!/usr/bin/env python
# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from minimal import SpanUser


class IntSpanTest(unittest.TestCase):

    def testCreateIntSpan(self):
        if not SpanUser.enabled():
            return
        expected = [1, 2, 3]
        self.assertEqual(SpanUser.getIntSpan3(), expected)
        self.assertEqual(SpanUser.getIntSpan(), expected)
        self.assertEqual(SpanUser.getConstIntSpan3(), expected)

        self.assertEqual(SpanUser.sumIntSpan3(expected), 6)
        self.assertEqual(SpanUser.sumIntSpan(expected), 6)
        self.assertEqual(SpanUser.sumConstIntSpan3(expected), 6)

    def testSpanOpaqueContainer(self):
        if not SpanUser.enabled():
            return
        oc = SpanUser.getIntSpan3_OpaqueContainer()  # 1,2,3
        oc[1] = 10
        oc = SpanUser.getIntSpan3_OpaqueContainer()
        # note: This converts to std::vector
        self.assertEqual(SpanUser.sumIntSpan3(oc), 14)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtCore import Qt, QPoint, QRect, QSize, Property, Slot
from PySide6.QtGui import QPainter, QPen
from PySide6.QtWidgets import QWidget


EMPTY = '-'
CROSS = 'X'
NOUGHT = 'O'
DEFAULT_STATE = "---------"


class TicTacToe(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self._state = DEFAULT_STATE
        self._turn_number = 0

    def minimumSizeHint(self):
        return QSize(200, 200)

    def sizeHint(self):
        return QSize(200, 200)

    def setState(self, new_state):
        self._turn_number = 0
        self._state = DEFAULT_STATE
        for position in range(min(9, len(new_state))):
            mark = new_state[position]
            if mark == CROSS or mark == NOUGHT:
                self._turn_number += 1
                self._change_state_at(position, mark)
            position += 1
        self.update()

    def state(self):
        return self._state

    @Slot()
    def clear_board(self):
        self._state = DEFAULT_STATE
        self._turn_number = 0
        self.update()

    def _change_state_at(self, pos, new_state):
        self._state = (self._state[:pos] + new_state
                       + self._state[pos + 1:])

    def mousePressEvent(self, event):
        if self._turn_number == 9:
            self.clear_board()
            return
        for position in range(9):
            cell = self._cell_rect(position)
            if cell.contains(event.position().toPoint()):
                if self._state[position] == EMPTY:
                    new_state = CROSS if self._turn_number % 2 == 0 else NOUGHT
                    self._change_state_at(position, new_state)
                    self._turn_number += 1
                    self.update()

    def paintEvent(self, event):
        with QPainter(self) as painter:
            painter.setRenderHint(QPainter.RenderHint.Antialiasing)

            painter.setPen(QPen(Qt.GlobalColor.darkGreen, 1))
            painter.drawLine(self._cell_width(), 0,
                             self._cell_width(), self.height())
            painter.drawLine(2 * self._cell_width(), 0,
                             2 * self._cell_width(), self.height())
            painter.drawLine(0, self._cell_height(),
                             self.width(), self._cell_height())
            painter.drawLine(0, 2 * self._cell_height(),
                             self.width(), 2 * self._cell_height())

            painter.setPen(QPen(Qt.GlobalColor.darkBlue, 2))

            for position in range(9):
                cell = self._cell_rect(position)
                if self._state[position] == CROSS:
                    painter.drawLine(cell.topLeft(), cell.bottomRight())
                    painter.drawLine(cell.topRight(), cell.bottomLeft())
                elif self._state[position] == NOUGHT:
                    painter.drawEllipse(cell)

            painter.setPen(QPen(Qt.GlobalColor.yellow, 3))

            for position in range(0, 8, 3):
                if (self._state[position] != EMPTY
                        and self._state[position + 1] == self._state[position]
                        and self._state[position + 2] == self._state[position]):
                    y = self._cell_rect(position).center().y()
                    painter.drawLine(0, y, self.width(), y)
                    self._turn_number = 9

            for position in range(3):
                if (self._state[position] != EMPTY
                        and self._state[position + 3] == self._state[position]
                        and self._state[position + 6] == self._state[position]):
                    x = self._cell_rect(position).center().x()
                    painter.drawLine(x, 0, x, self.height())
                    self._turn_number = 9

            if (self._state[0] != EMPTY and self._state[4] == self._state[0]
                    and self._state[8] == self._state[0]):
                painter.drawLine(0, 0, self.width(), self.height())
                self._turn_number = 9

            if (self._state[2] != EMPTY and self._state[4] == self._state[2]
                    and self._state[6] == self._state[2]):
                painter.drawLine(0, self.height(), self.width(), 0)
                self._turn_number = 9

    def _cell_rect(self, position):
        h_margin = self.width() / 30
        v_margin = self.height() / 30
        row = int(position / 3)
        column = position - 3 * row
        pos = QPoint(column * self._cell_width() + h_margin,
                     row * self._cell_height() + v_margin)
        size = QSize(self._cell_width() - 2 * h_margin,
                     self._cell_height() - 2 * v_margin)
        return QRect(pos, size)

    def _cell_width(self):
        return self.width() / 3

    def _cell_height(self):
        return self.height() / 3

    state = Property(str, state, setState)



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import os
from pathlib import Path
import sys
from PySide6.QtCore import QObject, QUrl
from PySide6.QtGui import QGuiApplication
from PySide6.QtQuick import QQuickView


def sayThis(s):
    print(s)


if __name__ == '__main__':
    app = QGuiApplication(sys.argv)
    view = QQuickView()
    qml_file = os.fspath(Path(__file__).resolve().parent / 'view.qml')
    view.setSource(QUrl.fromLocalFile(qml_file))
    if view.status() == QQuickView.Status.Error:
        sys.exit(-1)

    root = view.rootObject()
    button = root.findChild(QObject, "buttonMouseArea")
    button.clicked.connect(lambda: sayThis("clicked button (signal directly connected)"))

    view.show()
    res = app.exec()
    # Deleting the view before it goes out of scope is required to make sure all child QML instances
    # are destroyed in the correct order.
    del view
    sys.exit(res)



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import gc
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(True)

from testbinding import TestObject
from PySide6.QtCore import QObject, Signal, SignalInstance

'''Tests the behaviour of homonymous signals and slots.'''


class HomonymousSignalAndMethodTest(unittest.TestCase):

    def setUp(self):
        self.value = 123
        self.called = False
        self.obj = TestObject(self.value)

    def tearDown(self):
        del self.value
        del self.called
        del self.obj
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()

    def testIdValueSignalEmission(self):
        def callback(idValue):
            self.assertEqual(idValue, self.value)
        self.obj.idValue.connect(callback)
        self.obj.emitIdValueSignal()

    def testStaticMethodDoubleSignalEmission(self):
        def callback():
            self.called = True
        self.obj.staticMethodDouble.connect(callback)
        self.obj.emitStaticMethodDoubleSignal()
        self.assertTrue(self.called)

    def testSignalNotCallable(self):
        self.assertRaises(TypeError, self.obj.justASignal)

    def testCallingInstanceMethodWithArguments(self):
        self.assertRaises(TypeError, TestObject.idValue, 1)

    def testCallingInstanceMethodWithoutArguments(self):
        self.assertRaises(TypeError, TestObject.idValue)

    def testHomonymousSignalAndMethod(self):
        self.assertEqual(self.obj.idValue(), self.value)

    def testHomonymousSignalAndStaticMethod(self):
        self.assertEqual(TestObject.staticMethodDouble(3), 6)

    def testHomonymousSignalAndStaticMethodFromInstance(self):
        self.assertEqual(self.obj.staticMethodDouble(4), 8)


# PYSIDE-1730: Homonymous Methods with multiple inheritance

class Q(QObject):
    signal = Signal()

    def method(self):
        msg = 'Q::method'
        print(msg)
        return msg


class M:

    def signal(self):
        msg = 'M::signal'
        print(msg)
        return msg

    def method(self):
        msg = 'M::method'
        print(msg)
        return msg


class C(M, Q):

    def __init__(self):
        Q.__init__(self)
        M.__init__(self)


class HomonymousMultipleInheritanceTest(unittest.TestCase):

    def testHomonymousMultipleInheritance(self):
        c = C()
        self.assertEqual(c.method(), "M::method")  # okay
        self.assertEqual(c.signal(), "M::signal")  # problem on PySide6 6.2.2
        self.assertEqual(type(c.signal), SignalInstance)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2023 The Qt Company Ltd.
# Copyright (C) 2013 Riverbank Computing Limited.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the opengl/hellogl2 example from Qt v6.x"""

from argparse import ArgumentParser, RawTextHelpFormatter
import sys
from PySide6.QtCore import Qt
from PySide6.QtGui import QSurfaceFormat
from PySide6.QtWidgets import (QApplication, QMessageBox)


try:
    from mainwindow import MainWindow
    from glwidget import GLWidget
except ImportError:
    app = QApplication(sys.argv)
    message_box = QMessageBox(QMessageBox.Icon.Critical, "OpenGL hellogl",
                              "PyOpenGL must be installed to run this example.",
                              QMessageBox.StandardButton.Close)
    message_box.setDetailedText("Run:\npip install PyOpenGL PyOpenGL_accelerate")
    message_box.exec()
    sys.exit(1)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    parser = ArgumentParser(description="hellogl2",
                            formatter_class=RawTextHelpFormatter)
    parser.add_argument('--multisample', '-m', action='store_true',
                        help='Use Multisampling')
    parser.add_argument('--coreprofile', '-c', action='store_true',
                        help='Use Core Profile')
    parser.add_argument('--transparent', '-t', action='store_true',
                        help='Transparent Windows')
    options = parser.parse_args()

    fmt = QSurfaceFormat()
    fmt.setDepthBufferSize(24)
    if options.multisample:
        fmt.setSamples(4)
    if options.coreprofile:
        fmt.setVersion(3, 2)
        fmt.setProfile(QSurfaceFormat.OpenGLContextProfile.CoreProfile)
    QSurfaceFormat.setDefaultFormat(fmt)

    GLWidget.set_transparent(options.transparent)

    main_window = MainWindow()
    if options.transparent:
        main_window.setAttribute(Qt.WidgetAttribute.WA_TranslucentBackground)
        main_window.setAttribute(Qt.WidgetAttribute.WA_NoSystemBackground, False)

    main_window.show()

    res = app.exec()
    sys.exit(res)



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtCore import QLocale, QSignalBlocker, Slot
from PySide6.QtWidgets import QMainWindow
from PySide6.QtTextToSpeech import QTextToSpeech, QVoice

from ui_mainwindow import Ui_MainWindow


class MainWindow(QMainWindow):

    def __init__(self, parent=None):
        super().__init__(parent)

        self._speech = None
        self._voices = []

        self._ui = Ui_MainWindow()
        self._ui.setupUi(self)

        # Populate engine selection list
        self._ui.engine.addItem("Default", "default")
        engines = QTextToSpeech.availableEngines()
        for engine in engines:
            self._ui.engine.addItem(engine, engine)
        self._ui.engine.setCurrentIndex(0)
        self.engine_selected(0)

        self._ui.pitch.valueChanged.connect(self.set_pitch)
        self._ui.rate.valueChanged.connect(self.set_rate)
        self._ui.volume.valueChanged.connect(self.set_volume)
        self._ui.engine.currentIndexChanged.connect(self.engine_selected)
        self._ui.voice.currentIndexChanged.connect(self.voice_selected)
        self._ui.language.currentIndexChanged.connect(self.language_selected)

    @Slot(int)
    def set_rate(self, rate):
        self._speech.setRate(rate / 10.0)

    @Slot(int)
    def set_pitch(self, pitch):
        self._speech.setPitch(pitch / 10.0)

    @Slot(int)
    def set_volume(self, volume):
        self._speech.setVolume(volume / 100.0)

    @Slot(QTextToSpeech.State)
    def state_changed(self, state):
        if state == QTextToSpeech.Speaking:
            self._ui.statusbar.showMessage("Speech started...")
        elif state == QTextToSpeech.Ready:
            self._ui.statusbar.showMessage("Speech stopped...", 2000)
        elif state == QTextToSpeech.Paused:
            self._ui.statusbar.showMessage("Speech paused...")
        else:
            self._ui.statusbar.showMessage("Speech error!")

        self._ui.pauseButton.setEnabled(state == QTextToSpeech.Speaking)
        self._ui.resumeButton.setEnabled(state == QTextToSpeech.Paused)
        can_stop = state == QTextToSpeech.Speaking or state == QTextToSpeech.Paused
        self._ui.stopButton.setEnabled(can_stop)

    @Slot(int)
    def engine_selected(self, index):
        engine_name = self._ui.engine.itemData(index)
        self._speech = None
        self._speech = (QTextToSpeech(self) if engine_name == "default"
                        else QTextToSpeech(engine_name, self))

        # Block signals of the languages combobox while populating
        current = self._speech.locale()
        with QSignalBlocker(self._ui.language):
            self._ui.language.clear()
            # Populate the languages combobox before connecting its signal.
            locales = self._speech.availableLocales()
            for locale in locales:
                lang = QLocale.languageToString(locale.language())
                territory = QLocale.territoryToString(locale.territory())
                self._ui.language.addItem(f"{lang} ({territory})", locale)
                if locale.name() == current.name():
                    current = locale

        self.set_rate(self._ui.rate.value())
        self.set_pitch(self._ui.pitch.value())
        self.set_volume(self._ui.volume.value())

        self._ui.speakButton.clicked.connect(self.speak_text)
        self._ui.stopButton.clicked.connect(self.stop_speaking)
        self._ui.pauseButton.clicked.connect(self.pause_speaking)
        self._ui.resumeButton.clicked.connect(self._speech.resume)

        self._speech.stateChanged.connect(self.state_changed)
        self._speech.localeChanged.connect(self.locale_changed)

        self.locale_changed(current)

    @Slot()
    def speak_text(self):
        self._speech.say(self._ui.plainTextEdit.toPlainText())

    @Slot()
    def stop_speaking(self):
        self._speech.stop()

    @Slot()
    def pause_speaking(self):
        self._speech.pause()

    @Slot(int)
    def language_selected(self, language):
        locale = self._ui.language.itemData(language)
        self._speech.setLocale(locale)

    @Slot(int)
    def voice_selected(self, index):
        self._speech.setVoice(self._voices[index])

    @Slot(QLocale)
    def locale_changed(self, locale):
        self._ui.language.setCurrentIndex(self._ui.language.findData(locale))

        with QSignalBlocker(self._ui.voice):
            self._ui.voice.clear()
            self._voices = self._speech.availableVoices()
            current_voice = self._speech.voice()
            for voice in self._voices:
                name = voice.name()
                gender = QVoice.genderName(voice.gender())
                age = QVoice.ageName(voice.age())
                self._ui.voice.addItem(f"{name} - {gender} - {age}")
                if voice.name() == current_voice.name():
                    self._ui.voice.setCurrentIndex(self._ui.voice.count() - 1)



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QObject, Slot, QMetaObject


class MyObject(QObject):
    @Slot(int, str)
    def slot1(self, a, b):
        pass


class testAttribute(unittest.TestCase):
    def testBug896(self):
        mo = MyObject.staticMetaObject
        self.assertTrue(isinstance(mo, QMetaObject))
        self.assertEqual(mo.className(), 'MyObject')
        self.assertTrue(mo.indexOfSlot('slot1(int,QString)') > -1)

    def testDuplicateSlot(self):
        mo = MyObject.staticMetaObject
        self.assertEqual(mo.indexOfSignal('destroyed(void)'), -1)
        self.assertTrue(mo.indexOfSignal('destroyed()') > -1)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtUiTools import QUiLoader
from PySide6.QtCore import QFile
from PySide6.QtWidgets import QApplication, QWidget


app = QApplication([])
loader = QUiLoader()
file = Path(__file__).resolve().parent / 'bug_552.ui'
assert (file.is_file())
file = QFile(file)
w = QWidget()
# An exception can't be thrown
mainWindow = loader.load(file, w)



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import sys
from pathlib import Path

from PySide6.QtCore import QUrl
from PySide6.QtGui import QGuiApplication
from PySide6.QtQuick import QQuickView, QQuickWindow, QSGRendererInterface

from squircle import Squircle  # noqa: F401

if __name__ == "__main__":
    app = QGuiApplication(sys.argv)

    QQuickWindow.setGraphicsApi(QSGRendererInterface.GraphicsApi.OpenGL)

    view = QQuickView()
    view.setResizeMode(QQuickView.ResizeMode.SizeRootObjectToView)
    qml_file = Path(__file__).parent / "main.qml"
    view.setSource(QUrl.fromLocalFile(qml_file))

    if view.status() == QQuickView.Status.Error:
        sys.exit(-1)
    view.show()

    sys.exit(app.exec())



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for argument modification with more than nine arguments.'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import Point, applyHomogeneousTransform


class TransformTest(unittest.TestCase):
    '''Test cases for modifying a function with > 9 arguments.'''

    def testTransform_ValidMatrix(self):
        '''Transform applies successfully.'''
        p = Point(3, 4)
        r = applyHomogeneousTransform(p, 0, 1, 0, -1, 0, 0, 0, 0, 1)
        self.assertTrue(type(r) is Point)
        self.assertEqual(r.x(), 4)
        self.assertEqual(r.y(), -3)

    def testTransform_InvalidMatrix(self):
        '''Transform does not apply successfully.'''
        p = Point(3, 4)
        r = applyHomogeneousTransform(p, 1, 0, 0, 0, 1, 0, 0, 0, 0)
        self.assertTrue(r is None)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2024 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from PySide6.QtCharts import QChart, QChartView
from PySide6.QtGui import QKeyEvent, QMouseEvent
from PySide6.QtCore import QEvent, Qt
from PySide6.QtWidgets import QGraphicsView


class ChartView(QChartView):
    def __init__(self, chart, parent=None):
        super().__init__(chart, parent)

        self.setRubberBand(QChartView.RectangleRubberBand)
        self._isTouching = False

    def viewPortEvent(self, event: QEvent):

        if event.type() == QMouseEvent.TouchBegin:
            self._isTouching = True

            self.chart().setAnimationOptions(QChart.NoAnimation)

        return super().viewPortEvent(event)

    def mousePressEvent(self, event: QMouseEvent):

        if self._isTouching:
            return

        return super().mousePressEvent(event)

    def mouseMoveEvent(self, event: QMouseEvent):

        if self._isTouching:
            return

        return super().mouseMoveEvent(event)

    def mouseReleaseEvent(self, event: QMouseEvent):

        if self._isTouching:
            self._isTouching = False

        self.chart().setAnimationOptions(QChart.SeriesAnimations)

        return super().mouseReleaseEvent(event)

    def keyPressEvent(self, event: QKeyEvent):

        key = event.key()
        if key == Qt.Key_Plus:
            self.chart().zoomIn()

        elif key == Qt.Key_Minus:
            self.chart().zoomOut()

        elif key == Qt.Key_Left:
            self.chart().scroll(-10, 0)

        elif key == Qt.Key_Right:
            self.chart().scroll(10, 0)

        elif key == Qt.Key_Up:
            self.chart().scroll(0, 10)

        elif key == Qt.Key_Down:
            self.chart().scroll(0, -10)

        else:
            QGraphicsView.keyPressEvent(event)



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import gc
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QObject, Signal


class ExtQObject(QObject):

    mySignal = Signal()

    def __init__(self):
        super().__init__()


class UserSignalTest(unittest.TestCase):

    def setUp(self):
        self.emitter = ExtQObject()
        self.counter = 0

    def tearDown(self):
        del self.emitter
        del self.counter
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()

    def testConnectEmitDisconnect(self):

        def slot():
            self.counter += 1

        self.emitter.mySignal.connect(slot)

        self.assertEqual(self.counter, 0)
        self.emitter.mySignal.emit()
        self.assertEqual(self.counter, 1)
        self.emitter.mySignal.emit()
        self.assertEqual(self.counter, 2)

        self.emitter.mySignal.disconnect(slot)

        self.emitter.mySignal.emit()
        self.assertEqual(self.counter, 2)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the QtDBus listnames example from Qt v6.x"""

import sys
from PySide6.QtCore import QCoreApplication
from PySide6.QtDBus import QDBusConnection, QDBusInterface, QDBusReply


def method1():
    print("Method 1:")

    session_bus = QDBusConnection.sessionBus()
    reply = session_bus.interface().registeredServiceNames()
    if not reply.isValid():
        print("Error:", reply.error().message())
        sys.exit(1)
    values = reply.value()
    for name in values:
        print(name)


def method2():
    print("Method 2:")

    session_bus = QDBusConnection.sessionBus()
    dbus_iface = QDBusInterface("org.freedesktop.DBus", "/org/freedesktop/DBus",
                                "org.freedesktop.DBus", session_bus)
    message = dbus_iface.call("ListNames")
    reply = QDBusReply(message)
    print(reply.value())


def method3():
    print("Method 3:")
    session_bus = QDBusConnection.sessionBus()
    print(session_bus.interface().registeredServiceNames().value())


if __name__ == "__main__":
    app = QCoreApplication()

    if not QDBusConnection.sessionBus().isConnected():
        print("Cannot connect to the D-Bus session bus.\n"
              "To start it, run:\n"
              "\teval `dbus-launch --auto-syntax`\n",
              file=sys.stderr)
        sys.exit(1)

    method1()
    method2()
    method3()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the Threaded QOpenGLWidget Example from Qt v6.x"""

import sys

from argparse import ArgumentParser, RawTextHelpFormatter

from PySide6.QtWidgets import QApplication, QMessageBox
from PySide6.QtGui import QShortcut, QSurfaceFormat
from PySide6.QtCore import QCoreApplication, QPoint, qVersion, Qt

try:
    from OpenGL import GL
except ImportError:
    app = QApplication(sys.argv)
    message = "PyOpenGL must be installed to run this example."
    message_box = QMessageBox(QMessageBox.Icon.Critical,
                              "Threaded QOpenGLWidget Example",
                              message, QMessageBox.StandardButton.Close)
    detail = "Run:\npip install PyOpenGL PyOpenGL_accelerate"
    message_box.setDetailedText(detail)
    message_box.exec()
    sys.exit(1)

from glwidget import GLWidget
from mainwindow import MainWindow


if __name__ == "__main__":
    app = QApplication(sys.argv)

    desc = "Qt Threaded QOpenGLWidget Example"
    parser = ArgumentParser(description=desc,
                            formatter_class=RawTextHelpFormatter)
    parser.add_argument("--single", "-s", action="store_true",
                        help="Single thread")
    options = parser.parse_args()

    QCoreApplication.setApplicationName(desc)
    QCoreApplication.setOrganizationName("QtProject")
    QCoreApplication.setApplicationVersion(qVersion())

    format = QSurfaceFormat()
    format.setDepthBufferSize(16)
    QSurfaceFormat.setDefaultFormat(format)

    # Two top-level windows with two QOpenGLWidget children in each. The
    # rendering for the four QOpenGLWidgets happens on four separate threads.

    top_gl_widget = GLWidget()
    pos = top_gl_widget.screen().availableGeometry().topLeft()
    pos += QPoint(200, 200)
    top_gl_widget.setWindowTitle(desc + " top level")
    top_gl_widget.move(pos)
    top_gl_widget.show()

    functions = top_gl_widget.context().functions()
    vendor = functions.glGetString(GL.GL_VENDOR)
    renderer = functions.glGetString(GL.GL_RENDERER)
    gl_info = f"{vendor}/f{renderer}"

    supports_threading = ("nouveau" not in gl_info and "ANGLE" not in gl_info
                          and "llvmpipe" not in gl_info)
    tool_tip = gl_info
    if not supports_threading:
        tool_tip += "\ndoes not support threaded OpenGL."
    top_gl_widget.setToolTip(tool_tip)
    print(tool_tip)

    close_shortcut = QShortcut(Qt.Modifier.CTRL | Qt.Key.Key_Q, top_gl_widget)
    close_shortcut.activated.connect(QApplication.closeAllWindows)
    close_shortcut.setContext(Qt.ShortcutContext.ApplicationShortcut)

    mw1 = None
    mw2 = None

    if not options.single and supports_threading:
        pos += QPoint(100, 100)
        mw1 = MainWindow()
        mw1.setToolTip(tool_tip)
        mw1.move(pos)
        mw1.setWindowTitle(f"{desc} #1")
        mw1.show()
        pos += QPoint(100, 100)
        mw2 = MainWindow()
        mw2.setToolTip(tool_tip)
        mw2.move(pos)
        mw2.setWindowTitle(f"{desc} #2")
        mw2.show()

    sys.exit(app.exec())



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QTimer
from PySide6.QtWidgets import QSlider
from helper.usesqapplication import UsesQApplication


class TestBugPYSIDE189(UsesQApplication):

    def testDisconnect(self):
        # Disconnecting from a signal owned by a destroyed object
        # should raise an exception, not segfault.
        def onValueChanged(self, value):
            pass

        sld = QSlider()
        sld.valueChanged.connect(onValueChanged)

        sld.deleteLater()

        QTimer.singleShot(0, self.app.quit)
        self.app.exec()

        self.assertRaises(RuntimeError, sld.valueChanged.disconnect, onValueChanged)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2024 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import sys
from functools import partial
from dataclasses import dataclass

from PySide6.QtCore import Property, Signal, Slot
from PySide6.QtNetwork import QHttpHeaders
from PySide6.QtQml import QmlElement

from abstractresource import AbstractResource


tokenField = "token"
emailField = "email"
idField = "id"


QML_IMPORT_NAME = "ColorPalette"
QML_IMPORT_MAJOR_VERSION = 1


@QmlElement
class BasicLogin(AbstractResource):
    @dataclass
    class User:
        email: str
        token: bytes
        id: int

    userChanged = Signal()

    def __init__(self, parent=None):
        super().__init__(parent)
        self.m_user = None
        self.m_loginPath = ""
        self.m_logoutPath = ""
        self.m_user = None

    @Property(str, notify=userChanged)
    def user(self):
        return self.m_user.email if self.m_user else ""

    @Property(bool, notify=userChanged)
    def loggedIn(self):
        return bool(self.m_user)

    @Property(str)
    def loginPath(self):
        return self.m_loginPath

    @loginPath.setter
    def loginPath(self, p):
        self.m_loginPath = p

    @Property(str)
    def logoutPath(self):
        return self.m_logoutPath

    @logoutPath.setter
    def logoutPath(self, p):
        self.m_logoutPath = p

    @Slot("QVariantMap")
    def login(self, data):
        request = self.m_api.createRequest(self.m_loginPath)
        self.m_manager.post(request, data, self, partial(self.loginReply, data))

    def loginReply(self, data, reply):
        self.m_user = None
        if not reply.isSuccess():
            print("login: ", reply.errorString(), file=sys.stderr)
        (json, error) = reply.readJson()
        if json and json.isObject():
            json_object = json.object()
            if token := json_object.get(tokenField):
                email = data[emailField]
                token = json_object[tokenField]
                id = data[idField]
                self.m_user = BasicLogin.User(email, token, id)

        headers = QHttpHeaders()
        headers.append("token", self.m_user.token if self.m_user else "")
        self.m_api.setCommonHeaders(headers)
        self.userChanged.emit()

    @Slot()
    def logout(self):
        request = self.m_api.createRequest(self.m_logoutPath)
        self.m_manager.post(request, b"", self, self.logoutReply)

    def logoutReply(self, reply):
        if reply.isSuccess():
            self.m_user = None
            self.m_api.clearCommonHeaders()  # clears 'token' header
            self.userChanged.emit()
        else:
            print("logout: ", reply.errorString(), file=sys.stderr)



#!/usr/bin/env python
# Copyright (C) 2024 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

"""PYSIDE-2404: Test whether star imports work as they require special handling
   by the lazy initialization."""

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

SHIBOKEN_NAME = "shiboken6.Shiboken"
MINIMAL_NAME = "minimal"
OTHER_NAME = "other"

shiboken_loaded = 1 if sys.modules.get(SHIBOKEN_NAME) else 0
minimal_loaded = 1 if sys.modules.get(MINIMAL_NAME) else 0
other_loaded = 1 if sys.modules.get(OTHER_NAME) else 0

from minimal import *       # noqa: F403

shiboken_loaded += 2 if sys.modules.get(SHIBOKEN_NAME) else 0
minimal_loaded += 2 if sys.modules.get(MINIMAL_NAME) else 0
other_loaded += 2 if sys.modules.get(OTHER_NAME) else 0

from other import Number    # noqa: F403
from other import *         # noqa: F403

shiboken_loaded += 4 if sys.modules.get(SHIBOKEN_NAME) else 0
minimal_loaded += 4 if sys.modules.get(MINIMAL_NAME) else 0
other_loaded = +4 if sys.modules.get(OTHER_NAME) else 0

import shiboken6.Shiboken   # noqa: F401 F403

shiboken_loaded += 8 if sys.modules.get(SHIBOKEN_NAME) else 0


class ValTest(unittest.TestCase):

    def test(self):
        val_id = 123
        val = Val(val_id)  # noqa: F405
        self.assertEqual(val.valId(), val_id)


class Simple(Number):

    def __init__(self):
        Number.__init__(self, 42)


class OtherTest(unittest.TestCase):

    def testConstructor(self):
        o = Simple()
        self.assertTrue(isinstance(o, Number))


class StarImportTest(unittest.TestCase):
    """
    This test is meant for Lazy Init.
    We explicitly choose modules which are able to lazy load.

    The ValTest:
    ------------
    We load something with `import *`.
    There is no module from our known ones imported.
    This means we need stack introspection to find out that this was
    a star import and we must disable lazyness.

    The OtherTest:
    --------------
    We load something normally that should be lazy.
    After that, we follow with a star import.
    Now the stack introspection does not work, because the loading is
    cached. The first import did a lazy load. The following star import
    needs to undo the lazyness. But now we have a redirected import.

    All tests simply check if the objects are real and not just names.
    The <module>_loaded tests prevend upcoming internal dependencies.

    To make sure that Shiboken is really not involved, it is checked
    and really imported afterwards (ensuring nothing is misspelled).
    """

    def testStar(self):
        self.assertEqual(other_loaded, 4)
        self.assertEqual(minimal_loaded, 6)
        self.assertEqual(shiboken_loaded, 14)
        # Interesting effect: Did not expect that shiboken is loaded at all.


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QDnsLookup'''

import gc
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QCoreApplication
from PySide6.QtNetwork import QDnsLookup


class DnsLookupTestCase(unittest.TestCase):
    '''Test case for QDnsLookup'''

    def setUp(self):
        self._app = QCoreApplication([])
        self._lookup = QDnsLookup(QDnsLookup.Type.ANY, 'www.qt.io')
        self._lookup.finished.connect(self._finished)

    def tearDown(self):
        del self._lookup
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()

    def _finished(self):
        if self._lookup.error() == QDnsLookup.Error.NoError:
            nameRecords = self._lookup.canonicalNameRecords()
            if nameRecords:
                print(nameRecords[0].name())
        self._app.quit()

    def testLookup(self):
        self._lookup.lookup()
        self._app.exec()


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from helper.usesqapplication import UsesQApplication
from PySide6 import QtCore, QtGui, QtWidgets
from PySide6.QtWidgets import QMainWindow, QLabel, QWidget


def xprint(*args, **kw):
    if "-v" in sys.argv:
        print(*args, **kw)


# This is the original testcase of PYSIDE-1564
class Age:
    def __init__(self, age=0, **kwds):
        super().__init__(**kwds)

        self.age = age


class Person(QtCore.QObject, Age):
    def __init__(self, name, **kwds):
        super().__init__(**kwds)

        self.name = name


class OriginalMultipleInheritanceTest(unittest.TestCase):

    def testIt(self):
        xprint()
        p = Person("Joe", age=38)
        xprint(f"p.age = {p.age}")
        # This would crash if MI does not work.

# More tests follow:


# mro ('C', 'A', 'QObject', 'Object', 'B', 'object')
class A(QtCore.QObject):
    def __init__(self, anna=77, **kw):
        xprint(f'A: before init kw = {kw}')
        super().__init__(**kw)
        xprint('A: after init')


class B:
    def __init__(self, otto=6, age=7, **kw):
        xprint(f'B: before init kw = {kw}')
        if "killme" in kw:
            raise AssertionError("asdf")
        super().__init__(**kw)
        self.age = age
        xprint('B: after init')


class C(A, B):
    def __init__(self, **kw):
        xprint(f'C: before init kw = {kw}')
        super().__init__(**kw)
        xprint('C: after init')


# mro ('F', 'D', 'QCursor', 'E', 'QLabel', 'QFrame', 'QWidget', 'QObject',
#      'QPaintDevice', 'Object', 'object')
class D(QtGui.QCursor):
    def __init__(self, anna=77, **kw):
        xprint(f'D: before init kw = {kw}')
        super().__init__(**kw)
        xprint('D: after init')


class E:
    def __init__(self, age=7, **kw):
        xprint(f'E: before init kw = {kw}')
        super().__init__(**kw)
        self.age = age
        xprint('E: after init')


class F(D, E, QtWidgets.QLabel):
    def __init__(self, **kw):
        xprint(f'F: before init kw = {kw}')
        super().__init__(**kw)
        xprint('F: after init')


# mro ('I', 'G', 'QTextDocument', 'H', 'QLabel', 'QFrame', 'QWidget', 'QObject',
#      'QPaintDevice', 'Object', 'object')
# Similar, but this time we want to reach `H` without support from `super`.
class G(QtGui.QTextDocument):
    pass


class H:
    def __init__(self, age=7, **kw):
        xprint(f'H: before init kw = {kw}')
        super().__init__(**kw)
        self.age = age
        xprint('H: after init')


class II(G, H, QtWidgets.QLabel):
    pass


# PYSIDE-2294: Friedemann's test adapted.
#              We need to ignore positional args in mixin classes.
class Ui_X_MainWindow:  # Emulating uic
    def setupUi(self, MainWindow):
        MainWindow.resize(400, 300)
        self.lbl = QLabel(self)


class MainWindow(QMainWindow, Ui_X_MainWindow):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setupUi(self)


class AdditionalMultipleInheritanceTest(UsesQApplication):

    def testABC(self):
        xprint()
        res = C(otto=3, anna=5)
        self.assertEqual(res.age, 7)
        xprint()
        with self.assertRaises(AssertionError):
            res = C(killme=42)
        xprint()

    def testDEF(self):
        xprint()
        res = F(anna=5)
        self.assertEqual(res.age, 7)
        xprint()

    def testGHI(self):
        xprint()
        res = II(age=7)
        self.assertEqual(res.age, 7)
        xprint()

    def testParentDoesNotCrash(self):
        # This crashed with
        # TypeError: object.__init__() takes exactly one argument (the instance to initialize)
        MainWindow()


# PYSIDE-2654: Additional missing init test.
#              This must work if no __init__ is defined (Ui_Form)
class Ui_Form:
    pass


class Mixin:
    def __init__(self, **kwargs) -> None:
        super().__init__(**kwargs)


class Card(Mixin, QWidget):
    def __init__(self, parent=None) -> None:
        super().__init__(parent=parent)


class Demo(Card, Ui_Form):
    def __init__(self) -> None:
        super().__init__()


class MissingInitFunctionTest(UsesQApplication):
    def testMissing(self):
        Demo()
        # Tests if this works. Would crash without the extra
        # check for object.__init__


if __name__ == "__main__":
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtCore import Slot
from PySide6.QtWidgets import QDialog
from PySide6.QtSerialPort import QSerialPort

from ui_settingsdialog import Ui_SettingsDialog


class Settings:
    def __init__(self):
        self.parity = QSerialPort.Parity.EvenParity
        self.baud = QSerialPort.BaudRate.Baud19200
        self.data_bits = QSerialPort.DataBits.Data8
        self.stop_bits = QSerialPort.StopBits.OneStop
        self.response_time = 1000
        self.number_of_retries = 3


class SettingsDialog(QDialog):

    def __init__(self, parent):
        super().__init__(parent)
        self.m_settings = Settings()
        self.ui = Ui_SettingsDialog()
        self.ui.setupUi(self)

        self.ui.parityCombo.setCurrentIndex(1)
        self.ui.baudCombo.setCurrentText(f"{self.m_settings.baud}")
        self.ui.dataBitsCombo.setCurrentText(f"{self.m_settings.data_bits}")
        self.ui.stopBitsCombo.setCurrentText(f"{self.m_settings.stop_bits}")
        self.ui.timeoutSpinner.setValue(self.m_settings.response_time)
        self.ui.retriesSpinner.setValue(self.m_settings.number_of_retries)

        self.ui.applyButton.clicked.connect(self._apply)

    @Slot()
    def _apply(self):
        self.m_settings.parity = self.ui.parityCombo.currentIndex()
        if self.m_settings.parity > 0:
            self.m_settings.parity = self.m_settings.parity + 1
        self.m_settings.baud = int(self.ui.baudCombo.currentText())
        self.m_settings.data_bits = int(self.ui.dataBitsCombo.currentText())
        self.m_settings.stop_bits = int(self.ui.stopBitsCombo.currentText())
        self.m_settings.response_time = self.ui.timeoutSpinner.value()
        self.m_settings.number_of_retries = self.ui.retriesSpinner.value()

        self.hide()

    def settings(self):
        return self.m_settings



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QObject, Signal


class Sender(QObject):
    the_signal = Signal(int, int, int)


class ArgsDontMatch(unittest.TestCase):

    def callback(self, arg1):
        self.ok = True

    def testConnectSignalToSlotWithLessArgs(self):
        self.ok = False
        obj1 = Sender()
        obj1.the_signal.connect(self.callback)
        obj1.the_signal.emit(1, 2, 3)

        self.assertTrue(self.ok)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()
from minimal import (arrayFunc, arrayFuncInt, arrayFuncIntReturn,
                     arrayFuncIntReturnTypedef, arrayFuncIntTypedef,
                     arrayFuncReturn, arrayFuncReturnTypedef, arrayFuncTypedef)

try:
    import numpy as np
except ImportError as e:
    print(e)
    np = None


class TypedefTest(unittest.TestCase):

    def setUp(self):
        self.the_size = 8

    def test_arrayFuncInt(self):
        none = ()
        full = range(self.the_size)
        self.assertTrue(arrayFuncInt(none), "None is empty, arrayFuncInt should return true")
        self.assertFalse(arrayFuncInt(full), "Full is NOT empty, arrayFuncInt should return false")

        self.assertTrue(arrayFuncInt(np.array(none)),
                        "None is empty, arrayFuncInt should return true")
        self.assertFalse(arrayFuncInt(np.array(full)),
                         "Full is NOT empty, arrayFuncInt should return false")

    def test_arrayFuncIntTypedef(self):
        none = ()
        full = (1, 2, 3)
        self.assertTrue(arrayFuncIntTypedef(none),
                        "None is empty, arrayFuncIntTypedef should return true")
        self.assertFalse(arrayFuncIntTypedef(full),
                         "Full is NOT empty, arrayFuncIntTypedef should return false")

        self.assertTrue(arrayFuncIntTypedef(np.array(none)),
                        "None is empty, arrayFuncIntTypedef should return true")
        self.assertFalse(arrayFuncIntTypedef(np.array(full)),
                         "Full is NOT empty, arrayFuncIntTypedef should return false")

    def test_arrayFuncIntReturn(self):
        none = arrayFuncIntReturn(0)
        full = arrayFuncIntReturn(self.the_size)
        self.assertTrue((len(none) == 0), "none should be empty")
        self.assertTrue((len(full) == self.the_size),
                        f"full should have {self.the_size} elements")

    def test_arrayFuncIntReturnTypedef(self):
        none = arrayFuncIntReturnTypedef(0)
        full = arrayFuncIntReturnTypedef(self.the_size)
        self.assertTrue((len(none) == 0), "none should be empty")
        self.assertTrue((len(full) == self.the_size),
                        f"full should have {self.the_size} elements")

    def test_arrayFunc(self):
        none = ()
        full = range(self.the_size)
        self.assertTrue(arrayFunc(none), "None is empty, arrayFunc should return true")
        self.assertFalse(arrayFunc(full), "Full is NOT empty, arrayFunc should return false")

        self.assertTrue(arrayFunc(np.array(none)), "None is empty, arrayFunc should return true")
        self.assertFalse(arrayFunc(np.array(full)),
                         "Full is NOT empty, arrayFunc should return false")

    def test_arrayFuncTypedef(self):
        none = ()
        full = (1, 2, 3)
        self.assertTrue(arrayFuncTypedef(none),
                        "None is empty, arrayFuncTypedef should return true")
        self.assertFalse(arrayFuncTypedef(full),
                         "Full is NOT empty, arrayFuncTypedef should return false")

        self.assertTrue(arrayFuncTypedef(np.array(none)),
                        "None is empty, arrayFuncTypedef should return true")
        self.assertFalse(arrayFuncTypedef(np.array(full)),
                         "Full is NOT empty, arrayFuncTypedef should return false")

    def test_arrayFuncReturn(self):
        none = arrayFuncReturn(0)
        full = arrayFuncReturn(self.the_size)
        self.assertTrue((len(none) == 0), "none should be empty")
        self.assertTrue((len(full) == self.the_size),
                        f"full should have {self.the_size} elements")

    def test_arrayFuncReturnTypedef(self):
        none = arrayFuncReturnTypedef(0)
        full = arrayFuncReturnTypedef(self.the_size)
        self.assertTrue((len(none) == 0), "none should be empty")
        self.assertTrue((len(full) == self.the_size),
                        f"full should have {self.the_size} elements")


if __name__ == '__main__':
    if np is not None:
        unittest.main()



# Copyright (C) 2020 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import sys
from argparse import ArgumentParser, RawTextHelpFormatter

from PySide6.QtWidgets import (QApplication, QFileSystemModel,
                               QFileIconProvider, QScroller, QTreeView)
from PySide6.QtCore import QDir

"""PySide6 port of the widgets/itemviews/dirview example from Qt v6.x"""


if __name__ == "__main__":
    app = QApplication(sys.argv)

    name = "Dir View"
    argument_parser = ArgumentParser(description=name,
                                     formatter_class=RawTextHelpFormatter)
    argument_parser.add_argument("--no-custom", "-c", action="store_true",
                                 help="Set QFileSystemModel.DontUseCustomDirectoryIcons")
    argument_parser.add_argument("--no-watch", "-w", action="store_true",
                                 help="Set QFileSystemModel.DontWatch")
    argument_parser.add_argument("directory",
                                 help="The directory to start in.",
                                 nargs='?', type=str)
    options = argument_parser.parse_args()
    root_path = options.directory

    model = QFileSystemModel()
    icon_provider = QFileIconProvider()
    model.setIconProvider(icon_provider)
    model.setRootPath("")
    if options.no_custom:
        model.setOption(QFileSystemModel.DontUseCustomDirectoryIcons)
    if options.no_watch:
        model.setOption(QFileSystemModel.DontWatchForChanges)
    tree = QTreeView()
    tree.setModel(model)
    if root_path:
        root_index = model.index(QDir.cleanPath(root_path))
        if root_index.isValid():
            tree.setRootIndex(root_index)

    # Demonstrating look and feel features
    tree.setAnimated(False)
    tree.setIndentation(20)
    tree.setSortingEnabled(True)
    availableSize = tree.screen().availableGeometry().size()
    tree.resize(availableSize / 2)
    tree.setColumnWidth(0, tree.width() / 3)

    # Make it flickable on touchscreens
    QScroller.grabGesture(tree, QScroller.ScrollerGestureType.TouchGesture)

    tree.setWindowTitle(name)
    tree.show()

    sys.exit(app.exec())



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtWidgets import QComboBox
from PySide6.QtGui import QIntValidator
from PySide6.QtCore import Slot


class BitRateBox(QComboBox):

    def __init__(self, parent):
        super().__init__(parent)
        self.m_isFlexibleDataRateEnabled = False
        self.m_customSpeedValidator = None
        self.m_customSpeedValidator = QIntValidator(0, 1000000, self)
        self.fill_bit_rates()
        self.currentIndexChanged.connect(self.check_custom_speed_policy)

    def bit_rate(self):
        index = self.currentIndex()
        if index == self.count() - 1:
            return int(self.currentText)
        return int(self.itemData(index))

    def is_flexible_data_rate_enabled(self):
        return self.m_isFlexibleDataRateEnabled

    def set_flexible_date_rate_enabled(self, enabled):
        self.m_isFlexibleDataRateEnabled = enabled
        self.m_customSpeedValidator.setTop(10000000 if enabled else 1000000)
        self.fill_bit_rates()

    @Slot(int)
    def check_custom_speed_policy(self, idx):
        is_custom_speed = not self.itemData(idx)
        self.setEditable(is_custom_speed)
        if is_custom_speed:
            self.clearEditText()
            self.lineEdit().setValidator(self.m_customSpeedValidator)

    def fill_bit_rates(self):
        rates = [10000, 20000, 50000, 100000, 125000, 250000, 500000,
                 800000, 1000000]
        data_rates = [2000000, 4000000, 8000000]

        self.clear()
        for rate in rates:
            self.addItem(f"{rate}", rate)

        if self.is_flexible_data_rate_enabled():
            for rate in data_rates:
                self.addItem(f"{rate}", rate)

        self.addItem("Custom")
        self.setCurrentIndex(6)  # default is 500000 bits/sec



# Copyright (C) 2024 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from helper.usesqapplication import UsesQApplication


from PySide6.QtCore import QCoreApplication, QObject  # noqa: F401
from PySide6.QtQml import QQmlApplicationEngine, qmlRegisterType


class BaseClass(QObject):
    def __init__(self, p=None):
        super().__init__(p)


class ChildClass(BaseClass):
    def __init__(self, p=None):
        super().__init__(p)


class TestQmlRegisterType(UsesQApplication):
    """Test the legacy QML register functions."""

    def test(self):
        qmlRegisterType(BaseClass, 'test', 1, 0, 'BaseClass')
        qmlRegisterType(ChildClass, 'test', 1, 0, 'ChildClass')
        # PYSIDE-2709: qmlRegisterType() would set additional class info
        # on the meta objects for registration which caused another meta
        # object to be created, breaking inheritance.
        child = ChildClass()
        base = BaseClass()
        self.assertTrue(child.metaObject().inherits(base.metaObject()))

        engine = QQmlApplicationEngine()
        file = Path(__file__).resolve().parent / 'qmlregistertype_test.qml'

        engine.load(file)
        rootObjects = engine.rootObjects()
        self.assertTrue(rootObjects)
        self.assertTrue(type(rootObjects[0]), ChildClass)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QParallelAnimationGroup, QTimer
from PySide6.QtStateMachine import (QFinalState, QState, QStateMachine)

from helper.usesqapplication import UsesQApplication


class QStateMachineTest(UsesQApplication):

    def cb(self, *args):
        self.assertEqual(self.machine.defaultAnimations(), [self.anim])

    def testBasic(self):
        self.machine = QStateMachine()
        s1 = QState()
        s2 = QState()
        s3 = QFinalState()

        self.machine.started.connect(self.cb)

        self.anim = QParallelAnimationGroup()

        self.machine.addState(s1)
        self.machine.addState(s2)
        self.machine.addState(s3)
        self.machine.setInitialState(s1)
        self.machine.addDefaultAnimation(self.anim)
        self.machine.start()

        QTimer.singleShot(100, self.app.quit)
        self.app.exec()


class QSetConverterTest(UsesQApplication):
    '''Test converter of QSet toPython using QStateAnimation.configuration'''

    def testBasic(self):
        '''QStateMachine.configuration converting QSet to python set'''
        machine = QStateMachine()
        s1 = QState()
        machine.addState(s1)
        machine.setInitialState(s1)
        machine.start()

        QTimer.singleShot(100, self.app.quit)
        self.app.exec()

        configuration = machine.configuration()

        self.assertTrue(isinstance(configuration, set))
        self.assertTrue(s1 in configuration)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2024 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the Qt Hello Graphs example from Qt v6.x"""

from pathlib import Path
import sys
from PySide6.QtGui import QGuiApplication
from PySide6.QtQuick import QQuickView


if __name__ == '__main__':
    app = QGuiApplication(sys.argv)

    viewer = QQuickView()
    viewer.engine().addImportPath(Path(__file__).parent)
    viewer.setColor("black")
    viewer.loadFromModule("HelloGraphs", "Main")
    viewer.show()
    r = app.exec()
    del viewer
    sys.exit(r)



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import sys

from PySide6.QtCore import QPoint, QTimer, QTime, Qt
from PySide6.QtGui import (QGuiApplication, QPainter, QPainterStateGuard,
                           QPalette, QPolygon, QRasterWindow)

"""Simplified PySide6 port of the gui/analogclock example from Qt v6.x"""


class AnalogClockWindow(QRasterWindow):

    def __init__(self):
        super().__init__()
        self.setTitle("Analog Clock")
        self.resize(200, 200)

        self._timer = QTimer(self)
        self._timer.timeout.connect(self.update)
        self._timer.start(1000)

        self._hour_hand = QPolygon([QPoint(5, 14), QPoint(-5, 14),
                                    QPoint(-4, -71), QPoint(4, -71)])
        self._minute_hand = QPolygon([QPoint(4, 14), QPoint(-4, 14),
                                      QPoint(-3, -89), QPoint(3, -89)])
        self._seconds_hand = QPolygon([QPoint(1, 14), QPoint(-1, 14),
                                       QPoint(-1, -89), QPoint(1, -89)])

        palette = qApp.palette()  # noqa: F821
        self._background_color = palette.color(QPalette.ColorRole.Window)
        self._hour_color = palette.color(QPalette.ColorRole.Text)
        self._minute_color = palette.color(QPalette.ColorRole.Text)
        self._seconds_color = palette.color(QPalette.ColorRole.Accent)

    def paintEvent(self, e):
        with QPainter(self) as painter:
            self.render(painter)

    def render(self, painter):
        width = self.width()
        height = self.height()

        side = min(width, height)

        painter.fillRect(0, 0, width, height, self._background_color)
        painter.setRenderHint(QPainter.RenderHint.Antialiasing)
        painter.translate(width / 2, height / 2)
        painter.scale(side / 200.0, side / 200.0)

        time = QTime.currentTime()

        painter.setPen(Qt.PenStyle.NoPen)
        painter.setBrush(self._hour_color)

        with QPainterStateGuard(painter):
            painter.rotate(30.0 * ((time.hour() + time.minute() / 60.0)))
            painter.drawConvexPolygon(self._hour_hand)

        for _ in range(0, 12):
            painter.drawRect(73, -3, 16, 6)
            painter.rotate(30.0)

        painter.setBrush(self._minute_color)

        with QPainterStateGuard(painter):
            painter.rotate(6.0 * time.minute())
            painter.drawConvexPolygon(self._minute_hand)

        painter.setBrush(self._seconds_color)

        with QPainterStateGuard(painter):
            painter.rotate(6.0 * time.second())
            painter.drawConvexPolygon(self._seconds_hand)
            painter.drawEllipse(-3, -3, 6, 6)
            painter.drawEllipse(-5, -68, 10, 10)

        painter.setPen(self._minute_color)

        for _ in range(0, 60):
            painter.drawLine(92, 0, 96, 0)
            painter.rotate(6.0)


if __name__ == "__main__":
    app = QGuiApplication(sys.argv)
    clock = AnalogClockWindow()
    clock.show()
    sys.exit(app.exec())



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths  # noqa: E402
init_test_paths(False)

from helper.helper import quickview_errorstring  # noqa: E402
from helper.timedqguiapplication import TimedQGuiApplication  # noqa: E402

from PySide6.QtQuick import QQuickView  # noqa: E402
from PySide6.QtCore import QObject, Signal, Slot, QUrl  # noqa: E402
from PySide6.QtQml import QmlElement  # noqa: E402

QML_IMPORT_NAME = "test.Obj"
QML_IMPORT_MAJOR_VERSION = 1


@QmlElement
class Obj(QObject):
    def __init__(self):
        super().__init__()
        self.value = 0

    sumResult = Signal(int, name="sumResult", arguments=['sum'])

    @Slot(int, int)
    def sum(self, arg1, arg2):
        self.sumResult.emit(arg1 + arg2)

    @Slot(str)
    def sendValue(self, s):
        self.value = int(s)


class TestConnectionWithQml(TimedQGuiApplication):

    def testSignalArguments(self):
        view = QQuickView()
        obj = Obj()

        view.setInitialProperties({"o": obj})
        file = Path(__file__).resolve().parent / 'signal_arguments.qml'
        self.assertTrue(file.is_file())
        view.setSource(QUrl.fromLocalFile(file))
        root = view.rootObject()
        self.assertTrue(root, quickview_errorstring(view))
        button = root.findChild(QObject, "button")
        self.assertTrue(button)
        view.show()
        button.clicked.emit()
        self.assertEqual(obj.value, 42)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Helper metaclass do 'decorate' docstrings from base test case classes'''

import types


def copy_func(func):
    '''Helper function to copy a function object (except docstring)'''
    return types.FunctionType(func.func_code, func.func_globals, func.func_name,
                              func.func_defaults, func.func_closure)


class DocModifier(type):
    '''Metaclass for modifiying method documentation.

    It allows the managed class to modify the method documentation, adding
    prefixes and suffixes to a given set of methods.

    For example, you have some unittest.TestCases that run on different set
    of data. These methods could be written once in a base class and the
    inheriting class just setup different setUp/tearDown methods with the
    proper data. To differentiate the methods when using docstrings, you
    can add a suffix or prefix to the docstring.

    Variables in the implementing class:

    doc_prefix - The prefix to the docstring
    doc_suffix - The suffix to the docstring
    doc_filter - The function to filter the methods. If not provided, this
                 no docstrings are changed.'''

    def __new__(mcs, name, bases, dct):

        # FIXME currently we have to define doc_filter on each subclass
        filter_function = dct.get('doc_filter')
        if not filter_function:
            filter_function = lambda x: False  # noqa: E731

        for base in bases:
            for attr in [x for x in base.__dict__ if filter_function(x)]:
                original = getattr(base, attr)

                if original.__doc__:
                    copy = copy_func(original)
                    copy.__doc__ = (dct.get('doc_prefix', '')
                                    + original.__doc__
                                    + dct.get('doc_suffix', ''))
                    dct[attr] = copy

        return type.__new__(mcs, name, bases, dct)

    def __init__(mcs, name, bases, dct):
        super(DocModifier, mcs).__init__(name, bases, dct)


if __name__ == '__main__':

    # tests
    class BaseTest:
        __metaclass__ = DocModifier

        def testBase(self):
            '''base'''

        def testWithoutDoc(self):
            pass

    class Implementing(BaseTest):

        doc_filter = lambda x: x.startswith('test')  # noqa: E731
        doc_prefix = 'prefix'
        doc_suffix = 'suffix'

    class OnlyPrefix(BaseTest):

        doc_filter = lambda x: x.startswith('test')  # noqa: E731
        doc_prefix = 'prefix'

    class OnlySuffix(BaseTest):

        doc_filter = lambda x: x.startswith('test')  # noqa: E731
        doc_suffix = 'suffix'

    assert (Implementing.testBase.__doc__ == 'prefixbasesuffix')
    assert (Implementing.testWithoutDoc.__doc__ is None)
    assert (OnlySuffix.testBase.__doc__ == 'basesuffix')
    assert (OnlySuffix.testWithoutDoc.__doc__ is None)
    assert (OnlyPrefix.testBase.__doc__ == 'prefixbase')
    assert (OnlyPrefix.testWithoutDoc.__doc__ is None)



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QtAsyncio'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

import asyncio
from PySide6.QtAsyncio import QAsyncioEventLoopPolicy


class QAsyncioTestCase(unittest.TestCase):
    async def sleep(self, output):
        output += "Hello"
        await asyncio.sleep(0.2)
        output += "World"

    async def gather(self, output):
        await asyncio.gather(self.sleep(output), self.sleep(output), self.sleep(output))

    def test_sleep(self):
        outputs_expected = []
        outputs_real = []

        # Run the code without QAsyncioEventLoopPolicy
        asyncio.set_event_loop_policy(asyncio.DefaultEventLoopPolicy())
        asyncio.run(self.sleep(outputs_expected))

        # Run the code with QAsyncioEventLoopPolicy and QtEventLoop
        asyncio.set_event_loop_policy(QAsyncioEventLoopPolicy())
        asyncio.run(self.sleep(outputs_real))

        self.assertEqual(outputs_expected, outputs_real)

    def test_gather(self):
        outputs_expected = []
        outputs_real = []

        # Run the code without QAsyncioEventLoopPolicy
        asyncio.set_event_loop_policy(asyncio.DefaultEventLoopPolicy())
        asyncio.run(self.gather(outputs_expected))

        # Run the code with QAsyncioEventLoopPolicy and QtEventLoop
        asyncio.set_event_loop_policy(QAsyncioEventLoopPolicy())
        asyncio.run(self.gather(outputs_real))

        self.assertEqual(outputs_expected, outputs_real)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

"""
This is the now incorrect old version from Python 2.
It happens to work in another way and will be retained.
"""

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from helper.helper import quickview_errorstring

from PySide6.QtCore import Qt, QUrl, QTimer
from PySide6.QtGui import QGuiApplication, QPen
from PySide6.QtQml import qmlRegisterType
from PySide6.QtQuick import QQuickView, QQuickPaintedItem

paintCalled = False


class MetaA(type):
    pass


class A:
    __metaclass__ = MetaA


MetaB = type(QQuickPaintedItem)
B = QQuickPaintedItem


class MetaC(MetaA, MetaB):
    pass


class C(A, B):
    __metaclass__ = MetaC


class Bug825 (C):
    def __init__(self, parent=None):
        QQuickPaintedItem.__init__(self, parent)

    def paint(self, painter):
        global paintCalled
        pen = QPen(Qt.GlobalColor.black, 2)
        painter.setPen(pen)
        painter.drawPie(self.boundingRect(), 0, 128)
        paintCalled = True


class TestBug825 (unittest.TestCase):
    def testIt(self):
        global paintCalled
        app = QGuiApplication([])
        qmlRegisterType(Bug825, 'bugs', 1, 0, 'Bug825')
        self.assertRaises(TypeError, qmlRegisterType, A, 'bugs', 1, 0, 'A')

        view = QQuickView()
        file = Path(__file__).resolve().parent / 'bug_825.qml'
        self.assertTrue(file.is_file())
        view.setSource(QUrl.fromLocalFile(file))
        self.assertTrue(view.rootObject(), quickview_errorstring(view))
        view.show()
        QTimer.singleShot(250, view.close)
        app.exec()
        self.assertTrue(paintCalled)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtGui import QRadialGradient
from PySide6.QtCore import QPointF


class QRadialGradientConstructor(unittest.TestCase):
    def _compare(self, qptf, tpl):
        self.assertEqual((qptf.x(), qptf.y()), tpl)

    def _assertValues(self, grad):
        self._compare(grad.center(), (1.0, 2.0))
        self._compare(grad.focalPoint(), (3.0, 4.0))
        self.assertEqual(grad.radius(), 5.0)

    def testAllInt(self):
        grad = QRadialGradient(1, 2, 5, 3, 4)
        self._assertValues(grad)

    def testQPointF(self):
        grad = QRadialGradient(QPointF(1, 2), 5, QPointF(3, 4))
        self._assertValues(grad)

    def testSetQPointF(self):
        grad = QRadialGradient()
        grad.setCenter(QPointF(1, 2))
        self._compare(grad.center(), (1.0, 2.0))


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import re
import logging

from PySide6.QtCore import (QMargins, QRegularExpression, QRegularExpressionMatch,
                            QRegularExpressionMatchIterator, Qt, Slot,)
from PySide6.QtGui import (QAction, QColor, QContextMenuEvent, QFontDatabase,
                           QGuiApplication, QIcon, QPalette,)
from PySide6.QtWidgets import (QCheckBox, QComboBox, QDialog, QFormLayout,
                               QFrame, QGridLayout, QHBoxLayout, QLabel,
                               QLineEdit, QPlainTextEdit, QSpinBox,
                               QTreeWidget, QTreeWidgetItem, QVBoxLayout,
                               QWidget)


def rawStringLiteral(pattern: str) -> str:
    pattern = 'r"' + pattern
    pattern = pattern + '"'
    return pattern


def patternToCode(pattern: str) -> str:
    pattern = pattern.replace("\\", "\\\\")
    pattern = pattern.replace('"', '\\"')
    pattern = '"' + pattern
    pattern = pattern + '"'
    return pattern


def codeToPattern(code: str) -> str:

    try:
        _ = code[0]
    except IndexError:
        logging.warning("code is empty")
        return code

    code_characters = [c for c in code]
    index = 0
    code_characters_size = len(code_characters)
    while index < code_characters_size:
        if code_characters[index] == '\\':
            del code_characters[index]
            code_characters_size -= 1
        index += 1
    code = "".join(code_characters)

    if code.startswith('"') and code.endswith('"'):
        code = code[1:-1]  # remove quotes

    return code


def createHorizontalSeparator() -> QFrame:
    result = QFrame()
    result.setFrameStyle(QFrame.Shape.HLine | QFrame.Shadow.Sunken)
    return result


def createVerticalSeparator() -> QFrame:
    result = QFrame()
    result.setFrameStyle(QFrame.Shape.VLine | QFrame.Shadow.Sunken)
    return result


class PatternLineEdit(QLineEdit):
    def __init__(self, parent: QWidget = None):
        super().__init__(parent)
        self.escapeSelectionAction = QAction("Escape Selection", self)
        self.copyToCodeAction = QAction("Copy to Code", self)
        self.pasteFromCodeAction = QAction("Paste from Code", self)

        self.setClearButtonEnabled(True)
        self.escapeSelectionAction.triggered.connect(self.escapeSelection)
        self.copyToCodeAction.triggered.connect(self.copyToCode)
        self.pasteFromCodeAction.triggered.connect(self.pasteFromCode)

    @Slot()
    def escapeSelection(self):
        selection = self.selectedText()
        selection_start = self.selectionStart()
        escapedSelection = QRegularExpression.escape(selection)
        if escapedSelection != selection:
            t = self.text()
            t = (
                t[: selection_start]
                + escapedSelection
                + t[selection_start + len(selection):]
            )
            self.setText(t)

    @Slot()
    def copyToCode(self):
        QGuiApplication.clipboard().setText(patternToCode(self.text()))

    @Slot()
    def pasteFromCode(self):
        self.setText(codeToPattern(QGuiApplication.clipboard().text()))

    def contextMenuEvent(self, event: QContextMenuEvent) -> None:
        menu = self.createStandardContextMenu()
        menu.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)
        menu.addSeparator()
        self.escapeSelectionAction.setEnabled(self.hasSelectedText())
        menu.addAction(self.escapeSelectionAction)
        menu.addSeparator()
        menu.addAction(self.copyToCodeAction)
        menu.addAction(self.pasteFromCodeAction)
        menu.popup(event.globalPos())


class DisplayLineEdit(QLineEdit):
    def __init__(self, parent: QWidget = None):
        super().__init__(parent)
        self.setReadOnly(True)
        self.disablePalette: QPalette = self.palette()
        self.disablePalette.setBrush(
            QPalette.ColorRole.Base,
            self.disablePalette.brush(QPalette.ColorGroup.Disabled, QPalette.ColorRole.Base),
        )
        self.setPalette(self.disablePalette)

        self.copyAction = QAction(self)
        self.copyAction.setText("Copy to clipboard")
        self.copyAction.setIcon(QIcon(":/images/copy.png"))
        self.copyAction.triggered.connect(
            lambda: QGuiApplication.clipboard().setText(self.text())
        )
        self.addAction(self.copyAction, QLineEdit.ActionPosition.TrailingPosition)


class RegularExpressionDialog(QDialog):
    def __init__(self, parent: QWidget = None) -> None:
        super().__init__(parent)
        self.setupUi()
        self.setWindowTitle("QRegularExpression Example")
        self.regularExpression = QRegularExpression()
        self.patternLineEdit.textChanged.connect(self.refresh)
        self.subjectTextEdit.textChanged.connect(self.refresh)
        self.caseInsensitiveOptionCheckBox.toggled.connect(self.refresh)
        self.dotMatchesEverythingOptionCheckBox.toggled.connect(self.refresh)
        self.multilineOptionCheckBox.toggled.connect(self.refresh)
        self.extendedPatternSyntaxOptionCheckBox.toggled.connect(self.refresh)
        self.invertedGreedinessOptionCheckBox.toggled.connect(self.refresh)
        self.dontCaptureOptionCheckBox.toggled.connect(self.refresh)
        self.useUnicodePropertiesOptionCheckBox.toggled.connect(self.refresh)
        self.offsetSpinBox.valueChanged.connect(self.refresh)
        self.matchTypeComboBox.currentIndexChanged.connect(self.refresh)
        self.anchoredMatchOptionCheckBox.toggled.connect(self.refresh)
        self.dontCheckSubjectStringMatchOptionCheckBox.toggled.connect(self.refresh)

        self.patternLineEdit.setText("(\\+?\\d+)-(?P<prefix>\\d+)-(?P<number>\\w+)")
        self.subjectTextEdit.setPlainText(
            "My office number is +43-152-0123456, my mobile is 001-41-255512"
            " instead."
        )

        self.refresh()

    def setResultUiEnabled(self, enabled: bool):
        self.matchDetailsTreeWidget.setEnabled(enabled)
        self.namedGroupsTreeWidget.setEnabled(enabled)
        self.replacementTextEdit.setEnabled(enabled)

    def setTextColor(self, widget: QWidget, color: QColor):
        self.palette: QPalette = widget.palette()
        self.palette.setColor(QPalette.ColorRole.Text, color)
        widget.setPalette(self.palette)

    @Slot()
    def refresh(self):
        self.setUpdatesEnabled(False)
        self.pattern = self.patternLineEdit.text()
        self.text = self.subjectTextEdit.toPlainText()

        self.offsetSpinBox.setMaximum(max(0, len(self.text) - 1))

        self.escapedPatternLineEdit.setText(patternToCode(self.pattern))
        self.rawStringLiteralLineEdit.setText(rawStringLiteral(self.pattern))

        self.setTextColor(
            self.patternLineEdit,
            self.subjectTextEdit.palette().color(QPalette.ColorRole.Text),
        )
        self.matchDetailsTreeWidget.clear()
        self.namedGroupsTreeWidget.clear()
        self.regexpStatusLabel.setText("")
        self.replacementTextEdit.clear()

        if not self.pattern:
            self.setResultUiEnabled(False)
            self.setUpdatesEnabled(True)
            return

        self.regularExpression.setPattern(self.pattern)

        if not self.regularExpression.isValid():
            self.setTextColor(self.patternLineEdit, Qt.red)
            pattern_error_offset = self.regularExpression.patternErrorOffset()
            error_string = self.regularExpression.errorString()
            self.regexpStatusLabel.setText(
                "Invalid: syntax error at position"
                f" {pattern_error_offset}"
                f" ({error_string})"
            )
            self.setResultUiEnabled(False)
            self.setUpdatesEnabled(True)
            return

        self.setResultUiEnabled(True)
        matchType: QRegularExpression.MatchType = QRegularExpression.MatchType(
            self.matchTypeComboBox.currentData()
        )
        patternOptions = QRegularExpression.PatternOption.NoPatternOption
        matchOptions = QRegularExpression.MatchOption.NoMatchOption

        if self.anchoredMatchOptionCheckBox.isChecked():
            matchOptions |= QRegularExpression.MatchOption.AnchorAtOffsetMatchOption
        if self.dontCheckSubjectStringMatchOptionCheckBox.isChecked():
            matchOptions |= QRegularExpression.MatchOption.DontCheckSubjectStringMatchOption

        if self.caseInsensitiveOptionCheckBox.isChecked():
            patternOptions |= QRegularExpression.PatternOption.CaseInsensitiveOption
        if self.dotMatchesEverythingOptionCheckBox.isChecked():
            patternOptions |= QRegularExpression.PatternOption.DotMatchesEverythingOption
        if self.multilineOptionCheckBox.isChecked():
            patternOptions |= QRegularExpression.PatternOption.MultilineOption
        if self.extendedPatternSyntaxOptionCheckBox.isChecked():
            patternOptions |= QRegularExpression.PatternOption.ExtendedPatternSyntaxOption
        if self.invertedGreedinessOptionCheckBox.isChecked():
            patternOptions |= QRegularExpression.PatternOption.InvertedGreedinessOption
        if self.dontCaptureOptionCheckBox.isChecked():
            patternOptions |= QRegularExpression.PatternOption.DontCaptureOption
        if self.useUnicodePropertiesOptionCheckBox.isChecked():
            patternOptions |= QRegularExpression.PatternOption.UseUnicodePropertiesOption

        self.regularExpression.setPatternOptions(patternOptions)

        capturingGroupsCount = self.regularExpression.captureCount() + 1

        offset = self.offsetSpinBox.value()
        iterator: QRegularExpressionMatchIterator = self.regularExpression.globalMatch(
            self.text, offset, matchType, matchOptions
        )
        i = 0

        while iterator.hasNext():
            match: QRegularExpressionMatch = iterator.next()
            matchDetailTopItem = QTreeWidgetItem(self.matchDetailsTreeWidget)
            matchDetailTopItem.setText(0, str(i))

            for captureGroupIndex in range(capturingGroupsCount):
                matchDetailItem = QTreeWidgetItem(matchDetailTopItem)
                matchDetailItem.setText(1, str(captureGroupIndex))
                matchDetailItem.setText(2, match.captured(captureGroupIndex))

            i = i + 1

        self.matchDetailsTreeWidget.expandAll()

        self.regexpStatusLabel.setText("Valid")

        namedCaptureGroups = self.regularExpression.namedCaptureGroups()
        for i in range(len(namedCaptureGroups)):
            currentNamedCaptureGroup = namedCaptureGroups[i]
            namedGroupItem = QTreeWidgetItem(self.namedGroupsTreeWidget)
            namedGroupItem.setText(0, str(i))
            namedGroupItem.setText(
                1,
                "<no name>"
                if not currentNamedCaptureGroup
                else currentNamedCaptureGroup,
            )

        self.updateReplacement()
        self.setUpdatesEnabled(True)

    @Slot()
    def updateReplacement(self):
        self.replacementTextEdit.clear()
        replacement = self.replacementLineEdit.text()
        if not self.regularExpression.isValid() or not replacement:
            return

        current_text = self.subjectTextEdit.toPlainText()
        regular_expression_pattern = self.regularExpression.pattern()

        new_text = re.sub(regular_expression_pattern, replacement, current_text)
        self.replacementTextEdit.setPlainText(new_text)

    def setupUi(self):
        self.mainLayout = QVBoxLayout(self)
        self.mainLayout.addWidget(self.setupTextUi())
        self.mainLayout.addWidget(createHorizontalSeparator())

        self.horizontalLayout = QHBoxLayout()
        self.mainLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout.addWidget(self.setupOptionsUi())
        self.horizontalLayout.addWidget(createVerticalSeparator())
        self.horizontalLayout.addWidget(self.setupInfoUi())

        self._font = QFontDatabase.systemFont(QFontDatabase.SystemFont.FixedFont)
        self.patternLineEdit.setFont(self._font)
        self.rawStringLiteralLineEdit.setFont(self._font)
        self.escapedPatternLineEdit.setFont(self._font)
        self.replacementLineEdit.setFont(self._font)
        self.subjectTextEdit.setFont(self._font)
        self.replacementTextEdit.setFont(self._font)

    def setupOptionsUi(self):
        container = QWidget()

        form_layout = QFormLayout(container)
        form_layout.setFieldGrowthPolicy(QFormLayout.FieldGrowthPolicy.AllNonFixedFieldsGrow)
        form_layout.setContentsMargins(QMargins())

        form_layout.addRow(QLabel("<h3>Options</h3>"))

        self.caseInsensitiveOptionCheckBox = QCheckBox("Case insensitive (/i)")
        self.dotMatchesEverythingOptionCheckBox = QCheckBox("Dot matches everything (/s)")
        self.multilineOptionCheckBox = QCheckBox("Multiline (/m)")
        self.extendedPatternSyntaxOptionCheckBox = QCheckBox("Extended pattern (/x)")
        self.invertedGreedinessOptionCheckBox = QCheckBox("Inverted greediness")
        self.dontCaptureOptionCheckBox = QCheckBox("Don't capture")
        self.useUnicodePropertiesOptionCheckBox = QCheckBox("Use unicode properties (/u)")

        self.patternOptionsCheckBoxLayout = QGridLayout()
        gridRow = 0

        self.patternOptionsCheckBoxLayout.addWidget(self.caseInsensitiveOptionCheckBox, gridRow, 1)
        self.patternOptionsCheckBoxLayout.addWidget(
            self.dotMatchesEverythingOptionCheckBox, gridRow, 2)
        gridRow = gridRow + 1
        self.patternOptionsCheckBoxLayout.addWidget(self.multilineOptionCheckBox, gridRow, 1)
        self.patternOptionsCheckBoxLayout.addWidget(
            self.extendedPatternSyntaxOptionCheckBox, gridRow, 2)
        gridRow = gridRow + 1
        self.patternOptionsCheckBoxLayout.addWidget(
            self.invertedGreedinessOptionCheckBox, gridRow, 1)
        self.patternOptionsCheckBoxLayout.addWidget(self.dontCaptureOptionCheckBox, gridRow, 2)
        gridRow = gridRow + 1
        self.patternOptionsCheckBoxLayout.addWidget(
            self.useUnicodePropertiesOptionCheckBox, gridRow, 1)

        form_layout.addRow("Pattern options:", self.patternOptionsCheckBoxLayout)

        self.offsetSpinBox = QSpinBox()
        form_layout.addRow("Match &offset:", self.offsetSpinBox)

        self.matchTypeComboBox = QComboBox()
        self.matchTypeComboBox.addItem("Normal", QRegularExpression.MatchType.NormalMatch)
        self.matchTypeComboBox.addItem(
            "Partial prefer complete",
            QRegularExpression.MatchType.PartialPreferCompleteMatch,
        )
        self.matchTypeComboBox.addItem(
            "Partial prefer first", QRegularExpression.MatchType.PartialPreferFirstMatch
        )
        self.matchTypeComboBox.addItem("No match", QRegularExpression.MatchType.NoMatch)
        form_layout.addRow("Match &type:", self.matchTypeComboBox)

        self.dontCheckSubjectStringMatchOptionCheckBox = QCheckBox(
            "Don't check subject string"
        )
        self.anchoredMatchOptionCheckBox = QCheckBox("Anchored match")

        self.matchOptionsCheckBoxLayout = QGridLayout()
        self.matchOptionsCheckBoxLayout.addWidget(
            self.dontCheckSubjectStringMatchOptionCheckBox, 0, 0
        )
        self.matchOptionsCheckBoxLayout.addWidget(
            self.anchoredMatchOptionCheckBox, 0, 1
        )
        form_layout.addRow("Match options:", self.matchOptionsCheckBoxLayout)

        return container

    def setupInfoUi(self):
        container = QWidget()

        form_layout = QFormLayout(container)
        form_layout.setFieldGrowthPolicy(QFormLayout.FieldGrowthPolicy.AllNonFixedFieldsGrow)
        form_layout.setContentsMargins(QMargins())

        self.matchInfoLabel = QLabel("<h3>Match information</h3>")
        form_layout.addRow(self.matchInfoLabel)

        self.matchDetailsTreeWidget = QTreeWidget()

        self.matchDetailsTreeWidget.setHeaderLabels(
            ["Match index", "Group index", "Captured string"]
        )
        self.matchDetailsTreeWidget.setSizeAdjustPolicy(QTreeWidget.SizeAdjustPolicy.AdjustToContents)  # noqa: E501
        form_layout.addRow("Match details:", self.matchDetailsTreeWidget)

        form_layout.addRow(createHorizontalSeparator())

        self.regexpInfoLabel = QLabel("<h3>Regular expression information</h3>")
        form_layout.addRow(self.regexpInfoLabel)

        self.regexpStatusLabel = QLabel("Valid")
        self.regexpStatusLabel.setWordWrap(True)
        form_layout.addRow("Pattern status:", self.regexpStatusLabel)

        self.namedGroupsTreeWidget = QTreeWidget()
        self.namedGroupsTreeWidget.setHeaderLabels(["Index", "Named group"])
        self.namedGroupsTreeWidget.setSizeAdjustPolicy(QTreeWidget.SizeAdjustPolicy.AdjustToContents)  # noqa: E501
        self.namedGroupsTreeWidget.setRootIsDecorated(False)
        form_layout.addRow("Named groups:", self.namedGroupsTreeWidget)

        return container

    def setupTextUi(self):
        container = QWidget()
        form_layout = QFormLayout(container)
        form_layout.setFieldGrowthPolicy(QFormLayout.FieldGrowthPolicy.AllNonFixedFieldsGrow)
        form_layout.setContentsMargins(QMargins())

        self.regexpAndSubjectLabel = QLabel(
            "<h3>Regular expression and text input</h3>"
        )
        form_layout.addRow(self.regexpAndSubjectLabel)

        self.patternLineEdit = PatternLineEdit()
        self.patternLineEdit.setClearButtonEnabled(True)
        form_layout.addRow("&Pattern:", self.patternLineEdit)

        self.rawStringLiteralLineEdit = DisplayLineEdit()
        form_layout.addRow("&Raw string literal:", self.rawStringLiteralLineEdit)

        self.escapedPatternLineEdit = DisplayLineEdit()
        form_layout.addRow("&Escaped pattern:", self.escapedPatternLineEdit)

        self.subjectTextEdit = QPlainTextEdit()
        form_layout.addRow("&Subject text:", self.subjectTextEdit)

        form_layout.addRow(createHorizontalSeparator())

        self.replaceLabel = QLabel("<h3>Replacement")
        form_layout.addRow(self.replaceLabel)

        self.replacementLineEdit = QLineEdit()
        self.replacementLineEdit.setClearButtonEnabled(True)
        self.replacementLineEdit.textChanged.connect(self.updateReplacement)
        form_layout.addRow("&Replace by:", self.replacementLineEdit)
        self.replacementLineEdit.setToolTip(
            "Use \\1, \\2... as placeholders for the captured groups."
        )

        self.replacementTextEdit = QPlainTextEdit()
        self.replacementTextEdit.setReadOnly(True)
        form_layout.addRow("Result:", self.replacementTextEdit)

        return container



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtCore import Slot
from PySide6.QtGui import QAction, QIcon
from PySide6.QtWidgets import (QCheckBox, QComboBox, QDialog,
                               QGridLayout, QGroupBox, QHBoxLayout, QLabel,
                               QLineEdit, QMenu, QMessageBox, QPushButton,
                               QSpinBox, QStyle, QSystemTrayIcon, QTextEdit,
                               QVBoxLayout)

import rc_systray  # noqa: F401


class Window(QDialog):
    def __init__(self, parent=None):
        super().__init__(parent)

        self._icon_group_box = QGroupBox()
        self._icon_label = QLabel()
        self._icon_combo_box = QComboBox()
        self._show_icon_check_box = QCheckBox()

        self._message_group_box = QGroupBox()
        self._type_label = QLabel()
        self._duration_label = QLabel()
        self._duration_warning_label = QLabel()
        self._title_label = QLabel()
        self._body_label = QLabel()

        self._type_combo_box = QComboBox()
        self._duration_spin_box = QSpinBox()
        self._title_edit = QLineEdit()
        self._body_edit = QTextEdit()
        self._show_message_button = QPushButton()

        self._minimize_action = QAction()
        self._maximize_action = QAction()
        self._restore_action = QAction()
        self._quit_action = QAction()

        self._tray_icon = QSystemTrayIcon()
        self._tray_icon_menu = QMenu()

        self.create_icon_group_box()
        self.create_message_group_box()

        self._icon_label.setMinimumWidth(self._duration_label.sizeHint().width())

        self.create_actions()
        self.create_tray_icon()

        self._show_message_button.clicked.connect(self.show_message)
        self._show_icon_check_box.toggled.connect(self._tray_icon.setVisible)
        self._icon_combo_box.currentIndexChanged.connect(self.set_icon)
        self._tray_icon.messageClicked.connect(self.message_clicked)
        self._tray_icon.activated.connect(self.icon_activated)

        self._main_layout = QVBoxLayout()
        self._main_layout.addWidget(self._icon_group_box)
        self._main_layout.addWidget(self._message_group_box)
        self.setLayout(self._main_layout)

        self._icon_combo_box.setCurrentIndex(1)
        self._tray_icon.show()

        self.setWindowTitle("Systray")
        self.resize(400, 300)

    def setVisible(self, visible):
        self._minimize_action.setEnabled(visible)
        self._maximize_action.setEnabled(not self.isMaximized())
        self._restore_action.setEnabled(self.isMaximized() or not visible)
        super().setVisible(visible)

    def closeEvent(self, event):
        if not event.spontaneous() or not self.isVisible():
            return
        if self._tray_icon.isVisible():
            QMessageBox.information(self, "Systray",
                                    "The program will keep running in the system tray. "
                                    "To terminate the program, choose <b>Quit</b> in the context "
                                    "menu of the system tray entry.")
            self.hide()
            event.ignore()

    @Slot(int)
    def set_icon(self, index):
        icon = self._icon_combo_box.itemIcon(index)
        self._tray_icon.setIcon(icon)
        self.setWindowIcon(icon)
        self._tray_icon.setToolTip(self._icon_combo_box.itemText(index))

    @Slot(str)
    def icon_activated(self, reason):
        if reason == QSystemTrayIcon.Trigger:
            pass
        if reason == QSystemTrayIcon.DoubleClick:
            self._icon_combo_box.setCurrentIndex(
                (self._icon_combo_box.currentIndex() + 1) % self._icon_combo_box.count()
            )
        if reason == QSystemTrayIcon.MiddleClick:
            self.show_message()

    @Slot()
    def show_message(self):
        self._show_icon_check_box.setChecked(True)
        selected_icon = self._type_combo_box.itemData(self._type_combo_box.currentIndex())
        msg_icon = QSystemTrayIcon.MessageIcon(selected_icon)

        if selected_icon == -1:  # custom icon
            icon = QIcon(self._icon_combo_box.itemIcon(self._icon_combo_box.currentIndex()))
            self._tray_icon.showMessage(
                self._title_edit.text(),
                self._body_edit.toPlainText(),
                icon,
                self._duration_spin_box.value() * 1000,
            )
        else:
            self._tray_icon.showMessage(
                self._title_edit.text(),
                self._body_edit.toPlainText(),
                msg_icon,
                self._duration_spin_box.value() * 1000,
            )

    @Slot()
    def message_clicked(self):
        QMessageBox.information(None, "Systray",
                                "Sorry, I already gave what help I could.\n"
                                "Maybe you should try asking a human?")

    def create_icon_group_box(self):
        self._icon_group_box = QGroupBox("Tray Icon")

        self._icon_label = QLabel("Icon:")

        self._icon_combo_box = QComboBox()
        self._icon_combo_box.addItem(QIcon(":/images/bad.png"), "Bad")
        self._icon_combo_box.addItem(QIcon(":/images/heart.png"), "Heart")
        self._icon_combo_box.addItem(QIcon(":/images/trash.png"), "Trash")

        self._show_icon_check_box = QCheckBox("Show icon")
        self._show_icon_check_box.setChecked(True)

        icon_layout = QHBoxLayout()
        icon_layout.addWidget(self._icon_label)
        icon_layout.addWidget(self._icon_combo_box)
        icon_layout.addStretch()
        icon_layout.addWidget(self._show_icon_check_box)
        self._icon_group_box.setLayout(icon_layout)

    def create_message_group_box(self):
        self._message_group_box = QGroupBox("Balloon Message")

        self._type_label = QLabel("Type:")

        self._type_combo_box = QComboBox()
        self._type_combo_box.addItem("None", QSystemTrayIcon.MessageIcon.NoIcon)
        self._type_combo_box.addItem(
            self.style().standardIcon(QStyle.StandardPixmap.SP_MessageBoxInformation),
            "Information",
            QSystemTrayIcon.MessageIcon.Information,
        )
        self._type_combo_box.addItem(
            self.style().standardIcon(QStyle.StandardPixmap.SP_MessageBoxWarning),
            "Warning",
            QSystemTrayIcon.MessageIcon.Warning,
        )
        self._type_combo_box.addItem(
            self.style().standardIcon(QStyle.StandardPixmap.SP_MessageBoxCritical),
            "Critical",
            QSystemTrayIcon.MessageIcon.Critical,
        )
        self._type_combo_box.addItem(QIcon(), "Custom icon", -1)
        self._type_combo_box.setCurrentIndex(1)

        self._duration_label = QLabel("Duration:")

        self._duration_spin_box = QSpinBox()
        self._duration_spin_box.setRange(5, 60)
        self._duration_spin_box.setSuffix(" s")
        self._duration_spin_box.setValue(15)

        self._duration_warning_label = QLabel("(some systems might ignore this hint)")
        self._duration_warning_label.setIndent(10)

        self._title_label = QLabel("Title:")
        self._title_edit = QLineEdit("Cannot connect to network")
        self._body_label = QLabel("Body:")

        self._body_edit = QTextEdit()
        self._body_edit.setPlainText("Don't believe me. Honestly, I don't have a clue.\n"
                                     "Click this balloon for details.")

        self._show_message_button = QPushButton("Show Message")
        self._show_message_button.setDefault(True)

        message_layout = QGridLayout()
        message_layout.addWidget(self._type_label, 0, 0)
        message_layout.addWidget(self._type_combo_box, 0, 1, 1, 2)
        message_layout.addWidget(self._duration_label, 1, 0)
        message_layout.addWidget(self._duration_spin_box, 1, 1)
        message_layout.addWidget(self._duration_warning_label, 1, 2, 1, 3)
        message_layout.addWidget(self._title_label, 2, 0)
        message_layout.addWidget(self._title_edit, 2, 1, 1, 4)
        message_layout.addWidget(self._body_label, 3, 0)
        message_layout.addWidget(self._body_edit, 3, 1, 2, 4)
        message_layout.addWidget(self._show_message_button, 5, 4)
        message_layout.setColumnStretch(3, 1)
        message_layout.setRowStretch(4, 1)
        self._message_group_box.setLayout(message_layout)

    def create_actions(self):
        self._minimize_action = QAction("Minimize", self)
        self._minimize_action.triggered.connect(self.hide)

        self._maximize_action = QAction("Maximize", self)
        self._maximize_action.triggered.connect(self.showMaximized)

        self._restore_action = QAction("Restore", self)
        self._restore_action.triggered.connect(self.showNormal)

        self._quit_action = QAction("Quit", self)
        self._quit_action.triggered.connect(qApp.quit)  # noqa: F821

    def create_tray_icon(self):
        self._tray_icon_menu = QMenu(self)
        self._tray_icon_menu.addAction(self._minimize_action)
        self._tray_icon_menu.addAction(self._maximize_action)
        self._tray_icon_menu.addAction(self._restore_action)
        self._tray_icon_menu.addSeparator()
        self._tray_icon_menu.addAction(self._quit_action)

        self._tray_icon = QSystemTrayIcon(self)
        self._tray_icon.setContextMenu(self._tray_icon_menu)



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import pickle
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtGui import QColor


class TestQColor (unittest.TestCase):
    def reduceColor(self, c):
        p = pickle.dumps(c)
        c2 = pickle.loads(p)
        self.assertEqual(c.spec(), c2.spec())
        self.assertEqual(c, c2)

    def testReduceEmpty(self):
        self.reduceColor(QColor())

    def testReduceString(self):
        self.reduceColor(QColor('gray'))

    def testReduceRGB(self):
        self.reduceColor(QColor.fromRgbF(0.1, 0.2, 0.3, 0.4))

    def testReduceCMYK(self):
        self.reduceColor(QColor.fromCmykF(0.1, 0.2, 0.3, 0.4, 0.5))

    def testReduceHsl(self):
        self.reduceColor(QColor.fromHslF(0.1, 0.2, 0.3, 0.4))

    def testReduceHsv(self):
        self.reduceColor(QColor.fromHsvF(0.1, 0.2, 0.3, 0.4))


if __name__ == "__main__":
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from dataclasses import dataclass
from pathlib import Path
import sys
from PySide6.QtCore import QAbstractListModel, Qt, QByteArray
from PySide6.QtGui import QGuiApplication
from PySide6.QtQuick import QQuickView
from PySide6.QtQml import QmlElement, QmlSingleton


QML_IMPORT_NAME = "PersonModel"
QML_IMPORT_MAJOR_VERSION = 1


@dataclass
class Person:
    name: str
    myrole: str


@QmlElement
@QmlSingleton
class PersonModel (QAbstractListModel):
    MyRole = Qt.ItemDataRole.UserRole + 1

    def __init__(self, data, parent=None):
        super().__init__(parent)
        self._data = data

    def roleNames(self):
        roles = {
            PersonModel.MyRole: QByteArray(b'myrole'),
            Qt.ItemDataRole.DisplayRole: QByteArray(b'display')
        }
        return roles

    def rowCount(self, index):
        return len(self._data)

    def data(self, index, role):
        d = self._data[index.row()]
        if role == Qt.ItemDataRole.DisplayRole:
            return d.name
        if role == Qt.ItemDataRole.DecorationRole:
            return Qt.GlobalColor.black
        if role == PersonModel.MyRole:
            return d.myrole
        return None

    @staticmethod
    def create(engine):
        data = [Person('Qt', 'myrole'), Person('PySide', 'role2')]
        return PersonModel(data)


if __name__ == '__main__':
    app = QGuiApplication(sys.argv)
    view = QQuickView()
    view.setResizeMode(QQuickView.ResizeMode.SizeRootObjectToView)
    view.engine().addImportPath(Path(__file__).parent)
    view.loadFromModule("UsingModel", "Main")
    if view.status() == QQuickView.Status.Error:
        sys.exit(-1)
    view.show()

    r = app.exec()
    # Deleting the view before it goes out of scope is required to make sure all child QML instances
    # are destroyed in the correct order.
    del view
    sys.exit(r)



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtGui import (
    QMatrix2x2,
    QMatrix2x3,
    QMatrix2x4,
    QMatrix3x2,
    QMatrix3x3,
    QMatrix3x4,
    QMatrix4x2,
    QMatrix4x3,
    QMatrix4x4,
)


# Helper function to create sequential data for a matrix
def sequential_values(rows: int, cols: int) -> list[float]:
    return [float(i + 1) for i in range(rows * cols)]


class TestQMatrixIndexing(unittest.TestCase):

    def setUp(self):
        # Matrix types
        self.matrices = [
            (QMatrix2x2(sequential_values(2, 2)), 2, 2),
            (QMatrix2x3(sequential_values(2, 3)), 3, 2),
            (QMatrix2x4(sequential_values(2, 4)), 4, 2),
            (QMatrix3x2(sequential_values(3, 2)), 2, 3),
            (QMatrix3x3(sequential_values(3, 3)), 3, 3),
            (QMatrix3x4(sequential_values(3, 4)), 4, 3),
            (QMatrix4x2(sequential_values(4, 2)), 2, 4),
            (QMatrix4x3(sequential_values(4, 3)), 3, 4),
            (QMatrix4x4(sequential_values(4, 4)), 4, 4),
        ]

    def test_getitem(self):
        """Test [row, col] indexing for all matrix types."""
        for m, rows, cols in self.matrices:
            v = 1.0
            for x in range(rows):
                for y in range(cols):
                    self.assertEqual(m[x, y], v)
                    v += 1.0

    def test_callable_operator(self):
        """Test operator()(row, col) for all QMatrix types."""
        for m, rows, cols in self.matrices:
            v = 1.0
            for x in range(rows):
                for y in range(cols):
                    self.assertEqual(m(x, y), v)
                    v += 1.0


if __name__ == "__main__":
    unittest.main()



# Copyright (C) 2025 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths  # noqa: E402
init_test_paths(False)

from PySide6.QtGui import QGuiApplication, QNativeInterface  # noqa
from helper.usesqapplication import UsesQApplication  # noqa: E402


class TestNativeInterface(UsesQApplication):

    @unittest.skipUnless(sys.platform == "linux", "Linux only")
    def testLinuxNativeApplication(self):
        app = qApp  # noqa: F821
        native_app = app.nativeInterface()
        if native_app:
            if issubclass(type(native_app), QNativeInterface.QX11Application):
                self.assertTrue(native_app.display() != 0)
            elif issubclass(type(native_app), QNativeInterface.QWaylandApplication):
                self.assertTrue(native_app.display() != 0)
                self.assertTrue(native_app.seat() is not None)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for std::map container conversions'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import SampleNamespace
from shiboken_test_helper import objectFullname

from shibokensupport.signature import get_signature

# For tests of invisible namespaces, see
# enumfromremovednamespace_test.py / removednamespaces.h


class TestVariablesUnderNamespace(unittest.TestCase):
    def testIt(self):
        self.assertEqual(SampleNamespace.variableInNamespace, 42)


class TestClassesUnderNamespace(unittest.TestCase):
    def testIt(self):
        c1 = SampleNamespace.SomeClass()  # noqa F841
        e1 = SampleNamespace.SomeClass.ProtectedEnum()  # noqa F841
        c2 = SampleNamespace.SomeClass.SomeInnerClass()  # noqa F841
        e2 = SampleNamespace.SomeClass.SomeInnerClass.ProtectedEnum()  # noqa F841
        c3 = SampleNamespace.SomeClass.SomeInnerClass.OkThisIsRecursiveEnough()  # noqa F841
        e3 = SampleNamespace.SomeClass.SomeInnerClass.OkThisIsRecursiveEnough.NiceEnum(0)  # noqa F841

    def testFunctionAddedOnNamespace(self):
        res = SampleNamespace.ImInsideANamespace(2, 2)
        self.assertEqual(res, 4)

    def testTpNames(self):
        self.assertEqual(str(SampleNamespace.SomeClass),
                         "<class 'sample.SampleNamespace.SomeClass'>")
        self.assertEqual(str(SampleNamespace.SomeClass.ProtectedEnum),
                         "<enum 'ProtectedEnum'>")
        self.assertEqual(str(SampleNamespace.SomeClass.SomeInnerClass.ProtectedEnum),
                         "<enum 'ProtectedEnum'>")
        self.assertEqual(str(SampleNamespace.SomeClass.SomeInnerClass.OkThisIsRecursiveEnough),
                         "<class 'sample.SampleNamespace.SomeClass.SomeInnerClass.OkThisIsRecursiveEnough'>")  # noqa: E501
        self.assertEqual(str(SampleNamespace.SomeClass.SomeInnerClass.OkThisIsRecursiveEnough.NiceEnum),  # noqa: E501
                         "<enum 'NiceEnum'>")

        # Test if enum inside of class is correct represented
        an = objectFullname(get_signature(SampleNamespace.enumInEnumOut).parameters['in_'].annotation)  # noqa: E501
        self.assertEqual(an, "sample.SampleNamespace.InValue")
        an = objectFullname(get_signature(SampleNamespace.enumAsInt).parameters['value'].annotation)
        self.assertEqual(an, "sample.SampleNamespace.SomeClass.PublicScopedEnum")

    def testInlineNamespaces(self):
        cls = SampleNamespace.ClassWithinInlineNamespace()
        cls.setValue(SampleNamespace.EnumWithinInlineNamespace.EWIN_Value1)
        self.assertEqual(cls.value(), SampleNamespace.EnumWithinInlineNamespace.EWIN_Value1)

    def testEnumAlias(self):
        """Test whether an enumeration can be aliased to another one and values
           can be used interchangeably."""
        expected = SampleNamespace.SomeClass.OptionAlias.None_
        actual = SampleNamespace.SomeClass.passThroughOptionAlias(expected)
        self.assertEqual(expected, actual)
        actual = SampleNamespace.SomeClass.passThroughOption(expected)
        self.assertEqual(expected, actual)
        # The alias source values should also work
        actual = SampleNamespace.SomeClass.passThroughOptionAlias(SampleNamespace.Option.None_)
        self.assertEqual(expected, actual)
        actual = SampleNamespace.SomeClass.passThroughOption(SampleNamespace.Option.None_)
        self.assertEqual(expected, actual)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

# PySide6 tutorial 7


import sys

from PySide6.QtCore import Signal, Slot, Qt
from PySide6.QtGui import QFont
from PySide6.QtWidgets import (QApplication, QGridLayout, QLCDNumber,
                               QPushButton, QSlider, QVBoxLayout, QWidget)


class LCDRange(QWidget):

    value_changed = Signal(int)

    def __init__(self, parent=None):
        super().__init__(parent)

        lcd = QLCDNumber(2)

        self.slider = QSlider(Qt.Orientation.Horizontal)
        self.slider.setRange(0, 99)
        self.slider.setValue(0)

        self.slider.valueChanged.connect(lcd.display)
        self.slider.valueChanged.connect(self.value_changed)

        layout = QVBoxLayout(self)
        layout.addWidget(lcd)
        layout.addWidget(self.slider)

    def value(self):
        return self.slider.value()

    @Slot(int)
    def set_value(self, value):
        self.slider.setValue(value)


class MyWidget(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)

        quit = QPushButton("Quit")
        quit.setFont(QFont("Times", 18, QFont.Weight.Bold))
        quit.clicked.connect(qApp.quit)  # noqa: F821

        previous_range = None

        layout = QVBoxLayout(self)
        layout.addWidget(quit)
        grid = QGridLayout()
        layout.addLayout(grid)

        for row in range(3):
            for column in range(3):
                lcd_range = LCDRange()
                grid.addWidget(lcd_range, row, column)

                if previous_range:
                    lcd_range.value_changed.connect(previous_range.set_value)

                previous_range = lcd_range


if __name__ == '__main__':
    app = QApplication(sys.argv)
    widget = MyWidget()
    widget.show()
    sys.exit(app.exec())



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtCore import Property, QRunnable, Qt, Signal, Slot
from PySide6.QtQml import QmlElement
from PySide6.QtQuick import QQuickItem, QQuickWindow

from squirclerenderer import SquircleRenderer

# To be used on the @QmlElement decorator
# (QML_IMPORT_MINOR_VERSION is optional)
QML_IMPORT_NAME = "OpenGLUnderQML"
QML_IMPORT_MAJOR_VERSION = 1


class CleanupJob(QRunnable):
    def __init__(self, renderer):
        super().__init__()
        self._renderer = renderer

    def run(self):
        del self._renderer


@QmlElement
class Squircle(QQuickItem):

    tChanged = Signal()

    def __init__(self, parent=None):
        super().__init__(parent)
        self._t = 0.0
        self._renderer = None
        self.windowChanged.connect(self.handleWindowChanged)

    def t(self):
        return self._t

    def setT(self, value):
        if self._t == value:
            return
        self._t = value
        self.tChanged.emit()
        if self.window():
            self.window().update()

    @Slot(QQuickWindow)
    def handleWindowChanged(self, win):
        if win:
            win.beforeSynchronizing.connect(self.sync, type=Qt.ConnectionType.DirectConnection)
            win.sceneGraphInvalidated.connect(self.cleanup, type=Qt.ConnectionType.DirectConnection)
            win.setColor(Qt.GlobalColor.black)
            self.sync()

    @Slot()
    def cleanup(self):
        del self._renderer
        self._renderer = None

    @Slot()
    def sync(self):
        window = self.window()
        if not self._renderer:
            self._renderer = SquircleRenderer()
            window.beforeRendering.connect(self._renderer.init, Qt.ConnectionType.DirectConnection)
            window.beforeRenderPassRecording.connect(
                self._renderer.paint, Qt.ConnectionType.DirectConnection
            )
        self._renderer.setViewportSize(window.size() * window.devicePixelRatio())
        self._renderer.setT(self._t)
        self._renderer.setWindow(window)

    def releaseResources(self):
        self.window().scheduleRenderJob(
            CleanupJob(self._renderer), QQuickWindow.RenderStage.BeforeSynchronizingStage
        )
        self._renderer = None

    t = Property(float, t, setT, notify=tChanged)



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

''' Test bug 243: http://bugs.openbossa.org/show_bug.cgi?id=243'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtWidgets import QApplication, QMainWindow, QLayout


class QAppPresence(unittest.TestCase):

    def testBug(self):
        app = QApplication(sys.argv)  # noqa: F841
        window = QMainWindow()
        layout = window.layout()
        self.assertTrue(isinstance(layout, QLayout))


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

''' Test bug 389: http://bugs.openbossa.org/show_bug.cgi?id=389'''

import sys
import os
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from helper.usesqapplication import UsesQApplication
from PySide6.QtGui import QIcon
from PySide6.QtWidgets import QStyle, QWidget


class BugTest(UsesQApplication):
    def testCase(self):
        s = QWidget().style()
        i = s.standardIcon(QStyle.StandardPixmap.SP_TitleBarMinButton)
        self.assertEqual(type(i), QIcon)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2025 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

import unittest
import logging
import io
import sys
import os

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QMessageLogger, QLoggingCategory, QtMsgType


class QtMessageHandler(logging.Handler):
    def __init__(self, category):
        super().__init__()
        self.category = category

    def emit(self, record):
        log_entry = self.format(record)
        logger = QMessageLogger(__file__, record.lineno, record.funcName)

        if record.levelno == logging.DEBUG:
            if self.category.isDebugEnabled():
                logger.debug(self.category, log_entry)
            else:
                logger.debug(log_entry)


class TestQMessageLogger(unittest.TestCase):
    def setUp(self):
        self.logger = logging.getLogger("test_qmessagelogger")
        self.logger.setLevel(logging.DEBUG)
        self.stream = io.StringIO()
        self.capture_handler = logging.StreamHandler(self.stream)
        self.capture_handler.setLevel(logging.DEBUG)
        self.logger.addHandler(self.capture_handler)

    def tearDown(self):
        self.logger.removeHandler(self.capture_handler)

    def test_debug_with_category_enabled(self):
        category_enabled = QLoggingCategory("test.category.enabled")
        # 0 is QtDebugMsg
        category_enabled.setEnabled(QtMsgType.QtDebugMsg, True)

        qt_handler_enabled = QtMessageHandler(category_enabled)
        self.logger.addHandler(qt_handler_enabled)

        self.logger.debug("Debug with category enabled")
        self.logger.removeHandler(qt_handler_enabled)

        captured = self.stream.getvalue()
        self.assertIn("Debug with category enabled", captured)

    def test_debug_with_category_disabled(self):
        category_disabled = QLoggingCategory("test.category.disabled")

        qt_handler_disabled = QtMessageHandler(category_disabled)
        self.logger.addHandler(qt_handler_disabled)

        self.logger.debug("Debug with category disabled")
        self.logger.removeHandler(qt_handler_disabled)

        captured = self.stream.getvalue()
        self.assertIn("Debug with category disabled", captured)


if __name__ == "__main__":
    unittest.main()



# Copyright (C) 2013 Riverbank Computing Limited.
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the network/blockingfortunclient example from Qt v5.x, originating from PyQt"""

from PySide6.QtCore import (Signal, QDataStream, QMutex, QMutexLocker,
                            QThread, QWaitCondition)
from PySide6.QtGui import QIntValidator
from PySide6.QtWidgets import (QApplication, QDialogButtonBox, QGridLayout,
                               QLabel, QLineEdit, QMessageBox, QPushButton,
                               QWidget)
from PySide6.QtNetwork import (QAbstractSocket, QHostAddress, QNetworkInterface,
                               QTcpSocket)


class FortuneThread(QThread):
    new_fortune = Signal(str)

    error = Signal(int, str)

    def __init__(self, parent=None):
        super().__init__(parent)

        self.quit = False
        self._host_name = ''
        self.cond = QWaitCondition()
        self.mutex = QMutex()
        self.port = 0

    def __del__(self):
        self.mutex.lock()
        self.quit = True
        self.cond.wakeOne()
        self.mutex.unlock()
        self.wait()

    def request_new_fortune(self, hostname, port):
        with QMutexLocker(self.mutex):
            self._host_name = hostname
            self.port = port
            if not self.isRunning():
                self.start()
            else:
                self.cond.wakeOne()

    def run(self):
        self.mutex.lock()
        server_name = self._host_name
        server_port = self.port
        self.mutex.unlock()

        while not self.quit:
            timeout = 5 * 1000

            socket = QTcpSocket()
            socket.connectToHost(server_name, server_port)

            if not socket.waitForConnected(timeout):
                self.error.emit(socket.error(), socket.errorString())
                return

            while socket.bytesAvailable() < 2:
                if not socket.waitForReadyRead(timeout):
                    self.error.emit(socket.error(), socket.errorString())
                    return

            instr = QDataStream(socket)
            instr.setVersion(QDataStream.Qt_4_0)
            block_size = instr.readUInt16()

            while socket.bytesAvailable() < block_size:
                if not socket.waitForReadyRead(timeout):
                    self.error.emit(socket.error(), socket.errorString())
                    return

            self.mutex.lock()
            fortune = instr.readQString()
            self.new_fortune.emit(fortune)

            self.cond.wait(self.mutex)
            server_name = self._host_name
            server_port = self.port
            self.mutex.unlock()


class BlockingClient(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)

        self.thread = FortuneThread()
        self._current_fortune = ''

        host_label = QLabel("&Server name:")
        port_label = QLabel("S&erver port:")

        for ip_address in QNetworkInterface.allAddresses():
            if (ip_address != QHostAddress.SpecialAddress.LocalHost
                    and ip_address.toIPv4Address() != 0):
                break
        else:
            ip_address = QHostAddress(QHostAddress.SpecialAddress.LocalHost)

        ip_address = ip_address.toString()

        self._host_line_edit = QLineEdit(ip_address)
        self._port_line_edit = QLineEdit()
        self._port_line_edit.setValidator(QIntValidator(1, 65535, self))

        host_label.setBuddy(self._host_line_edit)
        port_label.setBuddy(self._port_line_edit)

        self._status_label = QLabel(
            "This example requires that you run the Fortune Server example as well.")
        self._status_label.setWordWrap(True)

        self._get_fortune_button = QPushButton("Get Fortune")
        self._get_fortune_button.setDefault(True)
        self._get_fortune_button.setEnabled(False)

        quit_button = QPushButton("Quit")

        button_box = QDialogButtonBox()
        button_box.addButton(self._get_fortune_button, QDialogButtonBox.ButtonRole.ActionRole)
        button_box.addButton(quit_button, QDialogButtonBox.ButtonRole.RejectRole)

        self._get_fortune_button.clicked.connect(self.request_new_fortune)
        quit_button.clicked.connect(self.close)
        self._host_line_edit.textChanged.connect(self.enable_get_fortune_button)
        self._port_line_edit.textChanged.connect(self.enable_get_fortune_button)
        self.thread.new_fortune.connect(self.show_fortune)
        self.thread.error.connect(self.display_error)

        main_layout = QGridLayout()
        main_layout.addWidget(host_label, 0, 0)
        main_layout.addWidget(self._host_line_edit, 0, 1)
        main_layout.addWidget(port_label, 1, 0)
        main_layout.addWidget(self._port_line_edit, 1, 1)
        main_layout.addWidget(self._status_label, 2, 0, 1, 2)
        main_layout.addWidget(button_box, 3, 0, 1, 2)
        self.setLayout(main_layout)

        self.setWindowTitle("Blocking Fortune Client")
        self._port_line_edit.setFocus()

    def request_new_fortune(self):
        self._get_fortune_button.setEnabled(False)
        self.thread.request_new_fortune(self._host_line_edit.text(),
                                        int(self._port_line_edit.text()))

    def show_fortune(self, nextFortune):
        if nextFortune == self._current_fortune:
            self.request_new_fortune()
            return

        self._current_fortune = nextFortune
        self._status_label.setText(self._current_fortune)
        self._get_fortune_button.setEnabled(True)

    def display_error(self, socketError, message):
        if socketError == QAbstractSocket.SocketError.HostNotFoundError:
            QMessageBox.information(self, "Blocking Fortune Client",
                                    "The host was not found. Please check the host and port "
                                    "settings.")
        elif socketError == QAbstractSocket.SocketError.ConnectionRefusedError:
            QMessageBox.information(self, "Blocking Fortune Client",
                                    "The connection was refused by the peer. Make sure the "
                                    "fortune server is running, and check that the host name "
                                    "and port settings are correct.")
        else:
            QMessageBox.information(self, "Blocking Fortune Client",
                                    f"The following error occurred: {message}.")

        self._get_fortune_button.setEnabled(True)

    def enable_get_fortune_button(self):
        self._get_fortune_button.setEnabled(self._host_line_edit.text() != ''
                                            and self._port_line_edit.text() != '')


if __name__ == '__main__':

    import sys

    app = QApplication(sys.argv)
    client = BlockingClient()
    client.show()
    sys.exit(app.exec())



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QHttp'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QUrl
from PySide6.QtNetwork import QNetworkAccessManager, QNetworkReply, QNetworkRequest
from helper.usesqapplication import UsesQApplication
from httpd import TestServer


class AccessManagerCase(UsesQApplication):

    def setUp(self):
        super(AccessManagerCase, self).setUp()
        self.httpd = TestServer()
        self.httpd.start()
        self.called = False

    def tearDown(self):
        super(AccessManagerCase, self).tearDown()
        if self.httpd:
            self.httpd.shutdown()
            self.httpd = None

    def goAway(self):
        self.httpd.shutdown()
        self.app.quit()
        self.httpd = None

    def slot_replyFinished(self, reply):
        self.assertEqual(type(reply), QNetworkReply)
        self.called = True
        self.goAway()

    def testNetworkRequest(self):
        manager = QNetworkAccessManager()
        manager.finished.connect(self.slot_replyFinished)
        port = self.httpd.port()
        manager.get(QNetworkRequest(QUrl(f"http://127.0.0.1:{port}")))
        self.app.exec()
        self.assertTrue(self.called)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for type discovery'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import (Abstract, Base1, Derived,
                    MDerived1, SonOfMDerived1, MDerived3)
from other import OtherMultipleDerived


class TypeDiscoveryTest(unittest.TestCase):

    def testPureVirtualsOfImpossibleTypeDiscovery(self):
        a = Derived.triggerImpossibleTypeDiscovery()
        self.assertEqual(type(a), Abstract)
        # call some pure virtual method
        a.pureVirtual()

    def testAnotherImpossibleTypeDiscovery(self):
        a = Derived.triggerAnotherImpossibleTypeDiscovery()
        self.assertEqual(type(a), Derived)

    def testMultipleInheritance(self):
        obj = OtherMultipleDerived.createObject("Base1")
        self.assertEqual(type(obj), Base1)
        # PYSIDE-868: In case of single line direct inheritance,
        # a factory function will return the class wrapper
        # of the derived class.
        obj = OtherMultipleDerived.createObject("MDerived1")
        self.assertEqual(type(obj), MDerived1)
        obj = OtherMultipleDerived.createObject("SonOfMDerived1")
        self.assertEqual(type(obj), SonOfMDerived1)
        obj = OtherMultipleDerived.createObject("MDerived3")
        self.assertEqual(type(obj), MDerived3)
        # PYSIDE-868: OtherMultipleDerived inherits
        # OtherBase, Base1. In this case, a factory
        # function will return the base class wrapper.
        obj = OtherMultipleDerived.createObject("OtherMultipleDerived")
        self.assertEqual(type(obj), Base1)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtWidgets import (QApplication, QGridLayout, QLabel, QVBoxLayout,
                               QWidget)


class BuggyWidget(QWidget):
    def setup(self):
        self.verticalLayout = QVBoxLayout(self)
        self.gridLayout = QGridLayout()
        self.lbl = QLabel(self)
        self.gridLayout.addWidget(self.lbl, 0, 1, 1, 1)

        # this cause a segfault during the ownership transfer
        self.verticalLayout.addLayout(self.gridLayout)


class LayoutTransferOwnerShip(unittest.TestCase):
    def testBug(self):
        app = QApplication([])  # noqa: F841
        w = BuggyWidget()
        w.setup()
        w.show()
        self.assertTrue(True)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QObject


get_counter = 0
set_counter = 0


class Descriptor:
    def __get__(self, obj, owner):
        global get_counter

        if not obj:
            return self

        get_counter += 1
        return obj.var

    def __set__(self, obj, value):
        global set_counter

        set_counter += 1
        obj.var = value


class FooBar(QObject):
    test = Descriptor()
    var = 0


class SetAndGetTestCases(unittest.TestCase):
    def setUp(self):
        global get_counter
        global set_counter

        get_counter = 0
        set_counter = 0

    def testSetMethod(self):
        global get_counter
        global set_counter

        f = FooBar()

        f.test = 1
        self.assertEqual(get_counter, 0)
        self.assertEqual(set_counter, 1)

        get_counter = 0
        set_counter = 0

    def testGetMethod(self):
        global get_counter
        global set_counter

        f = FooBar()
        f.test = 1
        set_counter = 0

        ret = f.test  # noqa: F841
        self.assertEqual(get_counter, 1)
        self.assertEqual(set_counter, 0)

        get_counter = 0
        set_counter = 0


if __name__ == "__main__":
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QObject
from PySide6.QtWidgets import QApplication, QWidget
from PySide6.QtUiTools import QUiLoader


class View_1(QWidget):

    def __init__(self):
        super().__init__()
        loader = QUiLoader()
        file = Path(__file__).resolve().parent / 'bug_552.ui'
        assert (file.is_file())
        widget = loader.load(os.fspath(file), self)
        self.children = []
        for child in widget.findChildren(QObject, None):
            self.children.append(child)
        self.t = widget.tabWidget
        self.t.removeTab(0)


app = QApplication([])
window = View_1()
window.show()

# If it doesn't crash it works :-)



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

# For self.original
import PySide6  # noqa
from PySide6.QtGui import QMatrix2x2, QMatrix2x3, QMatrix2x4
from PySide6.QtGui import QMatrix3x2, QMatrix3x3, QMatrix3x4
from PySide6.QtGui import QMatrix4x2, QMatrix4x3, QMatrix4x4
from PySide6.QtGui import QVector2D, QVector3D, QVector4D
from PySide6.QtGui import QTransform, QQuaternion


class ReprCopyHelper:
    def testCopy(self):
        copy = eval(self.original.__repr__())
        self.assertTrue(copy is not self.original)
        self.assertEqual(copy, self.original)


class QTransformReprCopy(ReprCopyHelper, unittest.TestCase):
    def setUp(self):
        self.original = QTransform(1, 2, 3, 4, 5, 6, 7, 8, 9)


class QQuaternionReprCopy(ReprCopyHelper, unittest.TestCase):
    def setUp(self):
        self.original = QQuaternion(1, 2, 3, 4)


class QVector2DReprCopy(ReprCopyHelper, unittest.TestCase):
    def setUp(self):
        self.original = QVector2D(1, 2)


class QVector3DReprCopy(ReprCopyHelper, unittest.TestCase):
    def setUp(self):
        self.original = QVector3D(1, 2, 3)


class QVector4DReprCopy(ReprCopyHelper, unittest.TestCase):
    def setUp(self):
        self.original = QVector4D(1, 2, 3, 4)


class QMatrix2x2ReprCopy(ReprCopyHelper, unittest.TestCase):
    def setUp(self):
        self.original = QMatrix2x2([1, 2, 3, 4])


class QMatrix2x3ReprCopy(ReprCopyHelper, unittest.TestCase):
    def setUp(self):
        self.original = QMatrix2x3([1, 2, 3, 4, 5, 6])


class QMatrix2x4ReprCopy(ReprCopyHelper, unittest.TestCase):
    def setUp(self):
        self.original = QMatrix2x4([1, 2, 3, 4, 5, 6, 7, 8])


class QMatrix3x2ReprCopy(ReprCopyHelper, unittest.TestCase):
    def setUp(self):
        self.original = QMatrix3x2([1, 2, 3, 4, 5, 6])


class QMatrix3x3ReprCopy(ReprCopyHelper, unittest.TestCase):
    def setUp(self):
        self.original = QMatrix3x3([1, 2, 3, 4, 5, 6, 7, 8, 9])


class QMatrix3x4ReprCopy(ReprCopyHelper, unittest.TestCase):
    def setUp(self):
        self.original = QMatrix3x4([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12])


class QMatrix4x2ReprCopy(ReprCopyHelper, unittest.TestCase):
    def setUp(self):
        self.original = QMatrix4x2([1, 2, 3, 4, 5, 6, 7, 8])


class QMatrix4x3ReprCopy(ReprCopyHelper, unittest.TestCase):
    def setUp(self):
        self.original = QMatrix4x3([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12])


class QMatrix4x4ReprCopy(ReprCopyHelper, unittest.TestCase):
    def setUp(self):
        self.original = QMatrix4x4([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16])


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QObject methods'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QObject, Signal, Slot, Qt


class Obj(QObject):
    signal = Signal()

    def empty(self):
        pass

    def emitSignal(self):
        self.signal.emit()


class Receiver(QObject):

    def __init__(self, parent=None):
        super().__init__(parent)
        self._count = 0

    def count(self):
        return self._count

    @Slot()
    def testSlot(self):
        self._count += 1


class ObjectNameCase(unittest.TestCase):
    '''Tests related to QObject object name'''

    def testSimple(self):
        # QObject.objectName(string)
        name = 'object1'
        obj = QObject()
        obj.setObjectName(name)

        self.assertEqual(name, obj.objectName())

    def testEmpty(self):
        # QObject.objectName('')
        name = ''
        obj = QObject()
        obj.setObjectName(name)

        self.assertEqual(name, obj.objectName())

    def testDefault(self):
        # QObject.objectName() default
        obj = QObject()
        self.assertEqual('', obj.objectName())

    def testUnicode(self):
        name = 'não'
        # FIXME Strange error on upstream when using equal(name, obj)
        obj = QObject()
        obj.setObjectName(name)
        self.assertEqual(obj.objectName(), name)

    def testUniqueConnection(self):
        obj = Obj()
        # On first connect, UniqueConnection returns True, and on the second
        # it must return False, and not a RuntimeError (PYSIDE-34)
        self.assertTrue(obj.signal.connect(obj.empty, Qt.ConnectionType.UniqueConnection))
        self.assertFalse(obj.signal.connect(obj.empty, Qt.ConnectionType.UniqueConnection))

    def testDisconnect(self):
        obj = Obj()
        receiver = Receiver()
        conn_id = obj.signal.connect(receiver.testSlot)
        self.assertTrue(conn_id)

        obj.emitSignal()
        self.assertEqual(receiver.count(), 1)

        obj.signal.disconnect(conn_id)
        obj.emitSignal()
        self.assertEqual(receiver.count(), 1)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Unit tests for QCollator'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QCollator, QLocale, Qt


class QCollatorTest(unittest.TestCase):
    def testState(self):
        c = QCollator()
        c.setCaseSensitivity(Qt.CaseSensitivity.CaseInsensitive)
        c.setLocale(QLocale.Language.German)

        print("compare a and b:", c.compare("a", "b"))

        self.assertEqual(c.caseSensitivity(), Qt.CaseSensitivity.CaseInsensitive)
        self.assertEqual(c.locale(), QLocale(QLocale.Language.German))

        c.setLocale(QLocale.Language.French)
        c.setNumericMode(True)
        c.setIgnorePunctuation(True)
        c.setLocale(QLocale.Language.NorwegianBokmal)

        self.assertEqual(c.caseSensitivity(), Qt.CaseSensitivity.CaseInsensitive)
        self.assertEqual(c.numericMode(), True)
        self.assertEqual(c.ignorePunctuation(), True)
        self.assertEqual(c.locale(), QLocale(QLocale.Language.NorwegianBokmal))


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause

from PySide6.QtCore import Qt
from PySide6.QtGui import QFont, QFontMetrics
from PySide6.QtWidgets import QStyle, QStyledItemDelegate
from PySide6.QtPdf import QPdfSearchModel


class SearchResultDelegate(QStyledItemDelegate):

    def __init__(self, parent=None):
        super().__init__(parent)

    def paint(self, painter, option, index):
        displayText = index.data()
        boldBegin = displayText.find("<b>") + 3
        boldEnd = displayText.find("</b>", boldBegin)
        if boldBegin >= 3 and boldEnd > boldBegin:
            page = index.data(QPdfSearchModel.Role.Page.value)
            pageLabel = f"Page {page}: "
            boldText = displayText[boldBegin:boldEnd]
            if option.state & QStyle.State_Selected:
                painter.fillRect(option.rect, option.palette.highlight())
            defaultFont = painter.font()
            fm = painter.fontMetrics()
            pageLabelWidth = fm.horizontalAdvance(pageLabel)
            yOffset = (option.rect.height() - fm.height()) / 2 + fm.ascent()
            painter.drawText(0, option.rect.y() + yOffset, pageLabel)
            boldFont = QFont(defaultFont)
            boldFont.setBold(True)
            boldWidth = QFontMetrics(boldFont).horizontalAdvance(boldText)
            prefixSuffixWidth = (option.rect.width() - pageLabelWidth - boldWidth) / 2
            painter.setFont(boldFont)
            painter.drawText(pageLabelWidth + prefixSuffixWidth, option.rect.y() + yOffset,
                             boldText)
            painter.setFont(defaultFont)
            suffix = fm.elidedText(displayText[boldEnd + 4:],
                                   Qt.TextElideMode.ElideRight, prefixSuffixWidth)
            painter.drawText(pageLabelWidth + prefixSuffixWidth + boldWidth,
                             option.rect.y() + yOffset, suffix)
            prefix = fm.elidedText(displayText[0:boldBegin - 3],
                                   Qt.TextElideMode.ElideLeft, prefixSuffixWidth)
            painter.drawText(pageLabelWidth + prefixSuffixWidth - fm.horizontalAdvance(prefix),
                             option.rect.y() + yOffset, prefix)
        else:
            super().paint(painter, option, index)



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(True)

from PySide6.QtCore import QObject

from testbinding import SharedPointerTestbench, QSharedPointer_QObject


def create_qobject(name):
    result = QObject()
    result.setObjectName(name)
    return result


class SharedPointerTests(unittest.TestCase):

    def testObjSharedPointer(self):
        p = SharedPointerTestbench.createSharedPointerQObject()
        self.assertEqual(p.objectName(), "TestObject")
        SharedPointerTestbench.printSharedPointerQObject(p)
        p = SharedPointerTestbench.createSharedPointerConstQObject()
        SharedPointerTestbench.printSharedPointerConstQObject(p)

    def testIntSharedPointer(self):
        p = SharedPointerTestbench.createSharedPointerInt(42)
        SharedPointerTestbench.printSharedPointerInt(p)

    def testConstruction(self):
        name1 = "CreatedQObject1"
        p1 = QSharedPointer_QObject(create_qobject(name1))
        self.assertTrue(p1)
        self.assertEqual(p1.objectName(), name1)

        p1.reset()
        self.assertFalse(p1)

        name2 = "CreatedQObject2"
        p1.reset(create_qobject(name2))
        self.assertTrue(p1)
        self.assertEqual(p1.objectName(), name2)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QUrl
from PySide6.QtWidgets import QApplication
from PySide6.QtWebEngineWidgets import QWebEngineView

from helper.usesqapplication import UsesQApplication


class TestQWebEngineCookieStore(UsesQApplication):
    def testBasicFilter(self):
        src_dir = Path(__file__).resolve().parent
        html_path = src_dir / "resources" / "index.html"
        view = QWebEngineView()
        cookie_store = view.page().profile().cookieStore()
        firstPartyUrlPaths = []

        def cookie_filter(request):
            nonlocal firstPartyUrlPaths
            firstPartyUrlPaths.append(Path(request.firstPartyUrl.toLocalFile()))
            return False

        cookie_store.setCookieFilter(cookie_filter)
        view.loadFinished.connect(self._slot_loaded)
        view.load(QUrl.fromLocalFile(html_path))
        view.show()
        self.app.exec()

        self.assertEqual(len(firstPartyUrlPaths), 2)
        self.assertListEqual(firstPartyUrlPaths, [html_path, html_path])

    def _slot_loaded(self):
        QApplication.quit()


if __name__ == '__main__':
    unittest.main()
